(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                              XQuery Engine                          *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: glx_http.ml,v 1.2 2003/06/30 20:47:38 simeon Exp $ *)

(* Module Glx_http
   Description:
     This module contains some stuff used to hook up Galax to the http
     client provided by Trevor.
*)

open Http

open Error

type http_method =
  | File of string
  | Http of (string * int * string)
(* LDAP not supported for now - Jerome *)
(*| Ldap of string *)

let fix_local r =
  match r with
  | None -> "/"
  | Some x -> "/" ^ x

let glx_decode_url s =
  match decode_url s with
  | None ->
      (File s)
  | Some ("file", host, port, s) ->
      if host = "" && port = None then
	let local = fix_local s in
	(File local)
      else
	raise (Query (URI_Error "File URL should not have a host or port number"))
  | (Some ("http", host, port, s)) ->
      let local = fix_local s in
      let new_port =
	match port with
	| None -> 80   (* Default port *)
	| Some portnum ->
	    begin
	      try
		(int_of_string portnum)
	      with
	      | _ ->
		  raise (Query (URI_Error (portnum ^ " is not a valid port number")))
	    end
      in 
      (Http (host, new_port, local))
  | (Some (me, _, _, _)) ->
      raise (Query (URI_Error ("Method: " ^ me ^ " not supported in URL")))

let glx_http_get server port local =
  HTTP.get server port local

(**
  * @param meth HTTP method (usually POST or GET)
  * @param request (SOAP request <-> message associated to the HTTP request)
  * @param uri
  *
**)
let http_soap request uri =
  let meth = "POST" in
  match glx_decode_url uri with
  | Http (host,port,local) ->
      HTTP.gen_request host port local meth 
	(Some ("Content-Type: text/xml; charset=utf-8\r\n", request))
  | File local ->
      raise (Query (URI_Error ("Method: file  not supported in SOAP server URL")))
	
