(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                              XQuery Engine                          *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: glx_pxp.ml,v 1.4 2003/05/13 16:54:01 simeon Exp $ *)

open Error

(* Module Glx_pxp
   Description:
     This module contains some stuff used to hook up Galax to PXP.
*)

(* Internal stuff *)

class warner =
  object 
    method warn w =
      print_endline ("WARNING: " ^ w)
  end

let global_warner = new warner

(**********************)
(* Character encoding *)
(**********************)

type encoding = Pxp_types.encoding
      (* All possible external encodings *)

type rep_encoding = Pxp_types.rep_encoding
      (* All possible internal encodings *)

(* Converting a string to an encoding *)

let encoding_of_string s =
  try
    Netconversion.encoding_of_string s
  with
  | _ ->
      raise (Query (Undefined ("Undefined character encoding: " ^ s)))

(* Converting an encoding to a string *)

let string_of_encoding =
  try
    Netconversion.string_of_encoding
  with
  | _ ->
      raise (Query (Undefined "Undefined character encoding"))

(* Converting an internal encoding to an encoding *)

let encoding_of_rep_encoding enc =
  match enc with
  | `Enc_utf8 -> `Enc_utf8
  | `Enc_iso88591 -> `Enc_iso88591

(* Converting an encoding to an internal encoding *)

let rep_encoding_of_encoding enc =
  match enc with
  | `Enc_utf8 -> `Enc_utf8
  | `Enc_iso88591 -> `Enc_iso88591
  | _ ->
      raise (Query (Undefined "Internal encoding can only be UTF8 or ISO-8859-1"))


(************************)
(* Smart lexing buffers *)
(************************)

type resolver =
    Pxp_reader.resolver

let resolve_as_file () =
  new Pxp_reader.resolve_as_file()

let resolve_as_string s =
  new Pxp_reader.resolve_read_this_string s

let init_resolver resolver encoding =
  begin
    resolver # init_rep_encoding encoding;
    let w = new warner in
    resolver # init_warner w
  end

let close_resolver resolver =
  resolver # close_all

let change_encoding resolver enc_string =
  try
    resolver # change_encoding enc_string
  with
  | _ -> 
      raise (Query (Undefined ("Undefined character encoding: " ^ enc_string)))
 
let open_resolver resolver fname =
  try
    resolver # open_in (Pxp_types.System fname)
  with
  | _ ->
      raise (Query (Undefined ("file: " ^ fname ^ " not found")))


(**********************************************)
(* String / character conversion capabilities *)
(**********************************************)

(* Converting a string from one encoding to another *)

let write_data_string from_enc to_enc s =
  try
    let b = Buffer.create 80 in
    Pxp_aux.write_data_string from_enc to_enc (`Out_buffer b) s;
    Buffer.contents b
  with
  | _ ->
      raise (Query (Undefined ("Cannot convert string \"" ^ s ^ "\" to output encoding")))

let write_markup_string from_enc to_enc s =
  try
    let b = Buffer.create 80 in
    Pxp_aux.write_markup_string from_enc to_enc (`Out_buffer b) s;
    Buffer.contents b
  with
  | _ ->
      raise (Query (Undefined ("Cannot convert markup name \"" ^ s ^ "\" to output encoding")))

let character enc k =
  Pxp_aux.character enc global_warner k

