(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                             XQuery Engine                           *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: whitespace.ml,v 1.8 2003/05/28 14:47:38 simeon Exp $ *)

(* Module Whitespace
   Description:
     Manipulation of whitespace within strings. Used during whitespace
     handling in XML/XQuery.
*)

(**************************)
(* Processing white space *)
(**************************)

let rec get_whitespace s b =
  try
    let c =
      Stream.next s
    in
    if (c = ' ' || c = '\n' || c = '\t' || c = '\r') then
      begin
	Buffer.add_char b c;
	get_whitespace s b
      end
    else
      let newb = Buffer.create 50 in
      begin
	Buffer.add_char newb c;
	let result = ((Buffer.contents b), (get_non_whitespace s newb)) in
	begin
	  Buffer.reset newb;
	  Buffer.clear newb;
	  result
	end
      end
  with
  | Stream.Failure ->
      ("","")

and get_non_whitespace s b =
  try
    let c =
      Stream.next s
    in
    if not(c = ' ' || c = '\n' || c = '\t' || c = '\r') then
      begin
	Buffer.add_char b c;
	get_non_whitespace s b
      end
    else
      let newb = Buffer.create 50 in
      begin
	Buffer.add_char newb c;
	let (lead,trail) = (get_whitespace s newb) in
	let result = (Buffer.contents b) ^ lead ^ trail in
	begin
	  Buffer.reset newb;
	  Buffer.clear newb;
	  result
	end
      end
  with
  | Stream.Failure ->
      (Buffer.contents b)

let whitespace_only text =
  let b = Buffer.create 50 in
  let s = Stream.of_string text in
  match (get_whitespace s b) with
  | (_,"") ->  true
  | (_,trail) -> false


let remove_whitespace_and_empty text =
  let b = Buffer.create 50 in
  let s = Stream.of_string text in
  let result =
    match (get_whitespace s b) with
    | (_,"") ->
	None
    | (_,trail) ->
	Some trail
  in
  begin
    Buffer.reset b;
    Buffer.clear b;
    result
  end


let remove_whitespace text =
  let b = Buffer.create 50 in
  let s = Stream.of_string text in
  match (get_whitespace s b) with
  | (_,trail) ->
      trail

(* Whitespace or not whitespace ? That is the question... *)

let white c =
  (c = ' ') || (c = '\n') || (c = '\t') || (c = '\r')

(* Whitespace preserving mode *)

type mode =
  | Preserve
  | Default

