(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                              XQuery Engine                          *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: export_context.ml,v 1.1 2003/06/09 15:28:26 simeon Exp $ *)

open Error

open Datamodel


(**********************)
(* The export context *)
(**********************)

type export_context =
    { mutable current_sequence : _Item list;
      stacked_nodes : (Sax_types.typed_sax_event * _Item list) Stack.t}


(********************************)
(* Creates a new export context *)
(********************************)

let build_export_context il =
  { current_sequence = il;
    stacked_nodes = Stack.create () }


(************************************)
(* Operations on the export context *)
(************************************)

(* Access the current sequence to stream *)

let get_current_sequence export_context =
  export_context.current_sequence

(* Replace the current sequence *)

let replace_current_sequence export_context new_sequence =
  export_context.current_sequence <- new_sequence

(* Push a context (getting inside of an element/document node) *)

let push_node_to_export_context export_context node remaining_list =
  let pop_event =
    match node with
    | DocumentNode _ -> Sax_types.TSAX_endDocument
    | ElementNode _ -> Sax_types.TSAX_endElement
    | AttributeNode _
    | TextNode _
    | NamespaceNode _
    | ProcessingInstructionNode _
    | CommentNode _ ->
	raise (Query (Internal_Error "Should never push a non-document and non-element node during datamodel to SAX streaming"))
  in
  Stack.push (pop_event, remaining_list) export_context.stacked_nodes

(* Pop a context (getting outside of an element/document node) *)

let pop_node_from_export_context export_context =
  try
    Some (Stack.pop export_context.stacked_nodes)
  with
  | Stack.Empty ->
      None

