(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                              XQuery Engine                          *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: export_datamodel.ml,v 1.1 2003/06/09 15:28:26 simeon Exp $ *)

open Error

open Sym

open Datamodel

open Export_context


(*********************************************)
(* Build a stream from a data model instance *)
(*********************************************)

(* Note:
     The following function turns a data model instance back into an
     XML stream.
   - Jerome *)

let typed_sax_attribute_of_attribute_node attribute =
  let attr_name    = attrName attribute in
  let attr_content = attrContent attribute in
  let attr_value   = list_of_sequence (attrValue attribute) in
  (attr_name,attr_content,attr_value)

let typed_sax_attributes_of_attribute_nodes attributes =
  List.map typed_sax_attribute_of_attribute_node attributes

let next_datamodel_event export_context =
  let (event,rest) =
    match get_current_sequence export_context with
    | [] ->
	begin
	  match pop_node_from_export_context export_context with
	  | Some (event,node_sequence) ->
	      (Some event,node_sequence)
	  | None ->
	      (None,[])
	end
    | item :: rest_of_sequence ->
	begin
	  match item with
	  | Node node ->
	      begin
		match node with
		| DocumentNode dnode ->
		    begin
		      push_node_to_export_context export_context node rest_of_sequence;
		      let doc_children = list_of_sequence (docChildren dnode) in
		      let sax_doc_children =
			List.map (fun x -> Node x) doc_children
		      in
		      (Some (Sax_types.TSAX_startDocument (None,None)), sax_doc_children)
		    end
		| ElementNode enode ->
		    begin
		      push_node_to_export_context export_context node rest_of_sequence;
		      let elem_name = elemName enode in
		      let elem_children = list_of_sequence (elemChildren enode) in
		      let elem_attributes = list_of_sequence (elemAttributes enode) in
		      let elem_value = list_of_sequence (elemValue enode) in
		      let sax_elem_children =
			List.map (fun x -> Node x) elem_children
		      in
		      let sax_elem_attributes =
			typed_sax_attributes_of_attribute_nodes elem_attributes
		      in
		      let has_element_content = elemHasElementContent enode in
		      let sax_elem =
			(elem_name,sax_elem_attributes,elem_value,has_element_content)
		      in
		      (Some (Sax_types.TSAX_startElement sax_elem),sax_elem_children)
		    end
		| AttributeNode anode ->
		    let sax_attribute = typed_sax_attribute_of_attribute_node anode in
		    (Some (Sax_types.TSAX_attribute sax_attribute), rest_of_sequence)
		| TextNode tnode ->
		    let sax_text = textContent tnode in
		    (Some (Sax_types.TSAX_characters sax_text), rest_of_sequence)
		| NamespaceNode nnode ->
		    (Some (Sax_types.TSAX_namespace), rest_of_sequence)
		| ProcessingInstructionNode pinode ->
		    let pi_target = piTarget pinode in
		    let pi_value  = piValue pinode in
		    (Some (Sax_types.TSAX_processingInstruction (pi_target,pi_value)), rest_of_sequence)
		| CommentNode cnode ->
		    let comment_value = commentValue cnode in
		    (Some (Sax_types.TSAX_comment comment_value), rest_of_sequence)
	      end
	  | AtomicValue atomic ->
	      (Some (Sax_types.TSAX_atomicValue atomic), rest_of_sequence)
	end
  in
  begin
    replace_current_sequence export_context rest;
    event
  end

let next_event export_context n =
  next_datamodel_event export_context

let typed_xml_stream_of_datamodel v =
  let export_context = build_export_context (list_of_sequence v) in
  (Stream.from (next_event export_context))

let typed_wf_xml_stream_of_datamodel n =
  let result_node = 
    if (sequenceIsSingle n) &&
      (isNode(sequenceGetSingle n)) && 
      (isElementNode(getNode(sequenceGetSingle n)) 
     || isDocumentNode(getNode(sequenceGetSingle n))) then
      getNode(sequenceGetSingle n)
    else
      let (atts, others) = filterAttributesFromSequence n in
      
      (* The cleanest way to serialize a sequence of items is to convert
         the sequence of items to a sequence of nodes, construct an xq:result
         element node whose content is this sequence, then serialize the
         new node. 
	 
         We apply the rules for converting sequences to nodes in Section 3.7.1. 
       *)
(*	    let atts' = (namespace_attributes Namespace.prefix_uri_table) @ atts in *)
      
      let xqresult = relem_symbol(Namespace.xq_prefix, (Namespace.xq_uri, "result")) in
      let nodes = value_to_node_sequence others in
      buildElementNode (elemNode xqresult sequenceEmpty atts nodes [])
  in
  typed_xml_stream_of_datamodel (sequenceBuildSingle (Node result_node))

