(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                              XQuery Engine                          *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: load_context.ml,v 1.2 2003/05/28 14:47:38 simeon Exp $ *)

(* Module: Load_context
   Description:
     This module implements a the loading context.
*)

open Namespace
open Datatypes

open Nodeid


(* A type for the loading context *)

type load_context =
    { load_nodeid_gen  : nodeid_gen;
      load_nsenv       : nsenv;
      load_base_uri    : xs_anyURI }

(* Creates a new loading context *)

let build_load_context base_uri =
  { load_nodeid_gen  = Nodeid.new_nodeid_gen ();
    load_nsenv       = Namespace.empty_nsenv ();
    load_base_uri    = base_uri }

(* Accesses the loading context *)

let get_nsenv load_context =
  load_context.load_nsenv

let get_nodeid_gen load_context =
  load_context.load_nodeid_gen

let get_base_uri load_context =
  load_context.load_base_uri

(* Operations on node identity *)

let new_prefix load_context =
  Nodeid.new_prefix load_context.load_nodeid_gen

let new_postfix load_context =
  Nodeid.new_postfix load_context.load_nodeid_gen

let new_leaf_nodeid load_context =
  Nodeid.new_leaf_nodeid load_context.load_nodeid_gen

let build_nodeid load_context prefix postfix =
  Nodeid.build_nodeid load_context.load_nodeid_gen prefix postfix

(* Adds namespace bindings to the loading context *)

let add_ns_bindings load_context bindings =
  let nsenv' = Namespace.add_all_ns load_context.load_nsenv bindings in
  { load_nodeid_gen  = load_context.load_nodeid_gen;
    load_nsenv       = nsenv';
    load_base_uri    = load_context.load_base_uri }

