(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                            XML Query Engine                         *)
(*                                                                     *)
(*  Copyright 2000, Lucent Technologies.                               *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: anyURI.ml,v 1.2 2002/12/10 16:50:37 simeon Exp $ *)

open Neturl

open Error

(* URI's are implemented based on the Netstring package *)

type _uri = url

(* URI syntax used *)

let url_syntax = ip_url_syntax

(* Resolves an absolute URI with a given URI scheme *)

let process_uri uri_string =
  try
    url_of_string url_syntax uri_string 
  with
  | _ ->
      raise (Query (URI_Error ("Mal formed URI " ^ uri_string)))

(* Default base URI and URI scheme *)

let default_scheme = "file"

let default_base_uri =
  let default_base_uri_string = "file:" ^ (Gmisc.rename_dir (Sys.getcwd ())) in
  process_uri default_base_uri_string

let base_uri_scheme =
  ref default_scheme

let base_uri_content =
  ref default_base_uri

let get_base_uri () =
  !base_uri_content

let set_base_uri uri_string =
  let scheme =
    try
      extract_url_scheme uri_string
    with
    | Malformed_URL ->
	raise (Query (URI_Error ("Mal formed base URI")))
  in
  base_uri_scheme := scheme;
  base_uri_content := process_uri uri_string

let _uri_of_string uri_string =
  process_uri uri_string

let _string_of_uri uri =
  string_of_url uri

let _uri_resolve base_uri relative_uri =
  apply_relative_url base_uri relative_uri

  (* Equality based on string values for now ? - Jerome *)
let _uri_eq uri1 uri2 =
  string_of_url uri1 = string_of_url uri2

