(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                            XML Query Engine                         *)
(*                                                                     *)
(*  Copyright 2000, Lucent Technologies.                               *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: datatypes_lexer.mll,v 1.1.2.1 2003/11/07 02:46:34 simeon Exp $ *)

(* Auxiliary lexical analyzer for XML Schema primitive types *)

{

open Decimal
open Lexing

open Error

}

(*****************************)
(* Lexer regular expressions *)
(*****************************)

(* Whitespace *)

let ws = [ ' ' '\t' '\r' '\n' ]

let whitespace = ws*
let whitespaceplus = ws+

(* Numerics *)

let digit = ['0'-'9']

let sign            = '-' | '+'
let exponent        = 'e' | 'E'

let integer_numeric = sign? digit+
let decimal_numeric = sign? ((digit+ ('.' digit*)?) | ('.' digit+))
let double_numeric  = decimal_numeric (exponent sign? digit+)?


(****************)
(* Lexing rules *)
(****************)

(* xs:int *)
rule parse_int = parse
  | whitespace integer_numeric whitespace eof
      { _int_of_string (Whitespace.remove_whitespace(lexeme lexbuf)) }

(* xs:integer *)
and parse_integer = parse
  | whitespace integer_numeric whitespace eof
      { _integer_of_string (Whitespace.remove_whitespace(lexeme lexbuf)) }

(* xs:decimal *)
and parse_decimal = parse
  | whitespace decimal_numeric whitespace eof
      { _decimal_of_string (Whitespace.remove_whitespace(lexeme lexbuf)) }

(* xs:float *)
and parse_float = parse
  | whitespace (double_numeric | "NaN" | "INF" | "-INF") whitespace eof
      { float_of_string (Whitespace.remove_whitespace(lexeme lexbuf)) }

(* xs:double *)
and parse_double = parse
  | whitespace (double_numeric | "NaN" | "INF" | "-INF") whitespace eof
      { float_of_string (Whitespace.remove_whitespace(lexeme lexbuf)) }

(* xs:boolean *)
and parse_boolean = parse
  | whitespace "true" whitespace eof
      { true }
  | whitespace "false" whitespace eof
      { false }
  | whitespace "1" whitespace eof
      { true }
  | whitespace "0" whitespace eof
      { false }

