open Str
open Lexing
open Gmisc 

(* define *)

let iso_datetime = Str.regexp "-?[0-9]+-[0-1][0-9]-[0-3][0-9]T[0-2][0-9]:[0-5][0-9]:[0-5][0-9]\(.[0-9]*\)?Z\|\(\+\|\-[0-2][0-9]:[0-5][0-9]\)";;

let iso_date = Str.regexp "\-?[0-9]+\-[0-1][0-9]\-[0-3][0-9]";;

let iso_time = Str.regexp "[0-2][0-9]:[0-5][0-9]:[0-5][0-9]\(\.[0-9]*\)?Z\|\(\+\|\-[0-2][0-9]:[0-5][0-9]\)";;

(* functions *)

let create_tuple some_list = 
  match some_list with 
    [a;b;c;] -> (a, b, c)
  | [a;b;] -> (a, b, "0")
  | _ -> failwith "Cannot preform list -> tuple operation";;


let return_leading_zero_to some_var correct_length = 
  if (String.length (string_of_int some_var) < correct_length) 
  then ("0" ^ string_of_int some_var) else (string_of_int some_var);;

(* types *)

type _date = {
  year : int;
  month : int;
  date : int;
}

type _time = {
  hours : int;
  minutes : int;
  seconds : float;
}

type timezone = {
  offset : _time;
  rel : int;
}

type _dateTime = {
  dt_date : _date;
  dt_time : _time;
  dt_timezone : timezone;
}

(* create functions *)

let create_date dt = 
  match dt with 
    (y, m, d) -> {
      year = int_of_string y; 
      month = int_of_string m;
      date = int_of_string d;
    };;

let create_time tm =
  match tm with 
    (h, m, s) -> {
      hours = int_of_string h;
      minutes = int_of_string m;
      seconds = float_of_string s;
    };;

let create_timezone tz = 
  match tz with 
    (r, time) -> {
      offset = time;
      rel = r;
    };;

(* Does this still work? -- yes it does *)
let create_datetime dt tm tz = {
  dt_date = dt;
  dt_time = tm;
  dt_timezone = tz;
}

(* return to string functions *)
let string_of_date date_var = ((string_of_int date_var.year) ^ "-" ^ (string_of_int date_var.month) ^ "-" ^ (string_of_int date_var.date))

let string_of_time time_var = ((string_of_int time_var.hours) ^ ":" ^ (string_of_int time_var.minutes) ^ ":" ^ (string_of_float time_var.seconds))

let string_of_timezone timezone_var = 
       	match timezone_var.rel with 
	1 -> ("+" ^ (string_of_int timezone_var.offset.hours) ^ ":" ^ (string_of_int timezone_var.offset.minutes));
	| -1 -> ("-" ^ (string_of_int timezone_var.offset.hours) ^ ":" ^ (string_of_int timezone_var.offset.minutes));
	| 0 -> "Z";
	| _ -> "Invalid relative";;

let string_of_dateTime datetime_var = 
    ((string_of_date datetime_var.dt_date) ^ "T" ^ (string_of_time datetime_var.dt_time) ^ (string_of_timezone datetime_var.dt_timezone))

(* print functions *)

let print_date date_var = 
	print_string ((string_of_int date_var.year) ^ "-" ^ (string_of_int date_var.month) ^ "-" ^ (string_of_int date_var.date))


let print_time time_var =
       	print_string ((string_of_int time_var.hours) ^ ":" ^ (string_of_int time_var.minutes) ^ ":" ^ (string_of_float time_var.seconds))

let print_timezone timezone_var = 
       	match timezone_var.rel with 
	1 -> print_string ("+" ^ (string_of_int timezone_var.offset.hours) ^ ":" ^ (string_of_int timezone_var.offset.minutes));
	| -1 -> print_string ("-" ^ (string_of_int timezone_var.offset.hours) ^ ":" ^ (string_of_int timezone_var.offset.minutes));
	| 0 -> print_string "Z";
	| _ -> print_string "Invalid relative";;

let print_datetime datetime_var = 
    print_date datetime_var.dt_date;
    print_string "T";
    print_time datetime_var.dt_time;
    print_timezone datetime_var.dt_timezone;;


(* Parse *)

let get_datetime d = 
try
	let i = search_forward iso_datetime d 0 in
	let dt1 = matched_string d in d
with Not_found -> d;;

let eval_timezone timetimezone = 
try
	let i = search_forward (regexp "Z\|+\|-") timetimezone 0 in
	let r = matched_string timetimezone in
        match r with
	"+" -> ((split_left_on_char timetimezone '+'),1)
	| "-" -> ((split_left_on_char timetimezone '-'),-1)
	| "Z" -> ((split_left_on_char timetimezone 'Z'),0)
	| _ -> failwith "no timezone"
with x -> raise x;;

let dateTime_of_string sample_dt = 
    let dt = get_datetime sample_dt in 
    let (str_date, str_timetz) = split_left_on_char dt 'T' in
    let real_date = create_date (create_tuple (Str.split (Str.regexp "-") str_date)) in
    let ((str_time, str_timezone), int_rel) = eval_timezone str_timetz in
    let real_time = create_time (create_tuple (split (regexp ":") str_time)) in
    let time_timezone = create_time (create_tuple (split (regexp ":") str_timezone)) in
    let real_timezone = create_timezone (int_rel, time_timezone) in
        create_datetime real_date real_time real_timezone
        
(* These functions are in the works *)

let date_of_string sample_date = 
    let (year, month, date) = (create_tuple (split (regexp "-") sample_date)) in
        create_date (year, month, date)

let time_of_string sample_time = 
    let (hour, mins, secs) = (create_tuple (split (regexp ":") sample_time)) in
        create_time (hour, mins, secs)    


(*
print_date real_date;;
print_newline;;
print_time real_time;; 
print_newline;;
print_timezone real_timezone;;
print_newline;;
print_datetime real_datetime;;
print_newline;;
*)

(* Operations *)

let dateTime_eq a b = 
  (string_of_dateTime a = string_of_dateTime b);;
  
  (*
  ((a.year = b.year) && (a.month = b.month) && (a.date = b.date) && (a.hours = b.hours) && (a.minutes = b.minutes) && (a.seconds = b.seconds));;
  *)

(* alternativily: large if-elseif-else block
if a.year > or = b.year then check if = then check if a.month > or = b.month, etc.
*)

let dateTime_gt a b = 
  (string_of_dateTime a > string_of_dateTime b);;

let dateTime_ge a b = 
  (string_of_dateTime a >= string_of_dateTime b);;

let dateTime_lt a b = 
  (string_of_dateTime a < string_of_dateTime b);;
  
let dateTime_le a b = 
  (string_of_dateTime a <= string_of_dateTime b);;
