(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                              XQuery Engine                          *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: eval_top.ml,v 1.2 2003/05/28 14:47:38 simeon Exp $ *)

(* Module Eval_top
   Description:
     This module implements evaluation of top-level statements and of
     the XQuery prolog.
*)

open Format

open Error

open Namespace
open Sym

open Datatypes
open Xquery_ast
open Datamodel

open Norm_context
open Static_context
open Dynamic_context

open Eval_expr
open Eval_update


(**************)
(* Statements *)
(**************)

let eval_cstatement dyn_ctxt cs =
  match cs.pcstatement_desc with
  | CSExpression ce ->
      eval_cexpr dyn_ctxt ce
  | CSUpdate cu ->
      eval_cupdate dyn_ctxt cu;
      sequenceEmpty

(**********)
(* Prolog *)
(**********)

(* Note:
     This part of the code deals with the evaluation of the query
     prolog.
   - Jerome *)
    
(* Global variables *)

let eval_var dyn_ctxt vd =
  match vd.pcvar_decl_desc with
  | (vn,e) ->
      let n = eval_cexpr dyn_ctxt e in
      add_var_value_to_dynamic_context dyn_ctxt (vn,n)

(* Global variables *)

let eval_key dyn_ctxt kd =
  match kd.pckey_def_desc with
  | (kn,e1,e2) ->
      let n1 = eval_cexpr dyn_ctxt e1 in

      let process_e2 dyn_ctxt n1 =
	let n2 =
	  (* Extends the dynamic environment for '.' *)
	  let dyn_ctxt' = add_var_value_to_dynamic_context dyn_ctxt (Xquery_astutil.dot, sequenceBuildSingle n1) in
	  (* Evaluates the sortspec expression *)
	  eval_cexpr dyn_ctxt' e2
	in
	let keyval =
	  try
	    if (sequenceIsEmpty n2) then 
	      ""
	    else
	      let i = (sequenceGetSingle n2) in
	      if (isNode i) then 
		let n = getNode i in
		nodeStringValue n
	      else 
		Datatypes.erase_atomic_value (getAtomicValue i)
	  with
	  | _ ->
	      raise (Query (Internal_Error "key definition: key value not an item"))
	in
	add_key_to_dynamic_context  dyn_ctxt (kn,keyval) (sequenceBuildSingle n1)
      in
      List.fold_left process_e2 dyn_ctxt (list_of_sequence n1)

(* Top-level declarations *)

let eval_toplevel dyn_ctxt q =
  let dyn_ctxt' = List.fold_left eval_var dyn_ctxt q.pcmodule_vars in
  List.fold_left eval_key dyn_ctxt' q.pcmodule_keys

(* Evaluate all queries *)

let rec eval_query dyn_ctxt (q : cquery_module) : (_Value list) =
  let dyn_ctxt' = eval_toplevel dyn_ctxt q in
  List.map (fun x -> eval_cstatement dyn_ctxt' x) q.pcmodule_exprs

