/***********************************************************************/
/*                                                                     */
/*                                 GALAX                               */
/*                              XQuery Engine                          */
/*                                                                     */
/*  Copyright 2001-2003, Lucent Technologies.                          */
/*  Distributed only by permission.                                    */
/*                                                                     */
/***********************************************************************/

/* $Id: glx_stub.c,v 1.6 2003/06/27 16:52:00 mff Exp $ */

#include <stdio.h>
#include <string.h>

#include "glx.h"

/**************************************/
/* Initialization of the Galax Engine */
/**************************************/

char *glx_error_string; 

void
glx_init()
{
  CAMLparam0();
  char *fake_argv[2];

#ifdef DEBUG
  fprintf(stderr,"Initializing Caml Env\n");
#endif

  fake_argv[0] = "galax";
  fake_argv[1] = 0;

  caml_startup(fake_argv);

#ifdef DEBUG
  fprintf(stderr,"Caml Env Initialized\n");
#endif

  CAMLreturn0;
}
/*********************/
/* Utility functions */
/*********************/
value 
camllistlist_from_itemlistlist (itemlist *args, int nargs) { 
  CAMLparam0();
  CAMLlocal4(caml_items, caml_list, cur, last);

  itemlist il; 
  int argct; 

  /* Convert C array of itemlists to Caml list of lists */
  /* This is not pretty! */
  if (nargs != 0) { 
    caml_list  = cur = alloc(2,0);
    last = Val_int(0); 
    for (argct = 0; argct < nargs; argct++) { 
      il = args[argct]; 
      caml_items = camllist_from_itemlist(il); 
      if (last != Val_int(0)) Store_field(last, 1, cur); 
      Store_field(cur, 0, caml_items); 
      last = cur; 
      cur = alloc(2,0); 
    }
    Store_field(last, 1, Val_int(0)); 
  } else caml_list = Val_int(0);

  CAMLreturn(caml_list);
}

value 
camlstrings_from_strings (char **args, int nargs) { 
  CAMLparam0();
  CAMLlocal3(caml_list, cur, last);

  int argct;
  char *arg; 

  /* Convert C array of strings to Caml list of strings */
  /* This is not pretty! */
  if (nargs != 0) { 
    caml_list  = cur = alloc(2,0);
    last = Val_int(0); 
    for (argct = 0; argct < nargs; argct++) { 
      arg = args[argct]; 
      if (last != Val_int(0)) Store_field(last, 1, cur); 
      Store_field(cur, 0, copy_string(arg)); 
      last = cur; 
      cur = alloc(2,0); 
    }
    Store_field(last, 1, Val_int(0)); 
  } else caml_list = Val_int(0);

  CAMLreturn(caml_list);
}
/*****************************/
/* Exceptions                */
/*****************************/

char *
glx_exception_string(value exn) 
{
  CAMLparam1(exn);  
  CAMLlocal1(caml_result);

  char * result_msg;

#ifdef DEBUG
  fprintf(stderr,"Trying to get the error message\n");
#endif

  //  static value * exn_string_closure = NULL;
  //  if (exn_string_closure == NULL) {
  //    /* First time around, look up by name */
  //    exn_string_closure = caml_named_value("glx_exception_string");
  //  }
#ifdef DEBUG
  fprintf(stderr,"Calling back exception conversion\n");
#endif

  //  caml_result = callback(*exn_string_closure, exn);
  result_msg = strdup(format_caml_exception(exn));

#ifdef DEBUG
  fprintf(stderr,"Found error message: %s!\n",result_msg);
#endif

  CAMLreturn(result_msg); 
}
/*****************************/
/* Atomic value constructors */
/*****************************/
 
int
glx_xs_string(char* temp, item *itp) 
{
  CAMLparam0();  /* This function takes no Caml value arguments */
  CAMLlocal1(caml_result);

  static value * glx_xs_string_closure = NULL;
  if (glx_xs_string_closure == NULL) {
    /* First time around, look up by name */
    glx_xs_string_closure = caml_named_value("glx_xs_string");
  }	
  caml_result = callback_exn(*glx_xs_string_closure, copy_string(temp));

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}

int
glx_xs_boolean(int i, item *itp)
{
  CAMLparam0();  /* This function takes no Caml value arguments */
  CAMLlocal1(caml_result);

  static value * glx_xs_boolean_closure = NULL;
  if (glx_xs_boolean_closure == NULL) {
    /* First time around, look up by name */
    glx_xs_boolean_closure = caml_named_value("glx_xs_boolean");
  }	
  caml_result = callback_exn(*glx_xs_boolean_closure, Val_bool(i));

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}

int
glx_xs_int(int i, item *itp) 
{
  CAMLparam0();  /* This function takes no Caml value arguments */
  CAMLlocal1(caml_result);
  
  static value * glx_xs_int_closure = NULL;
 if (glx_xs_int_closure == NULL) {
    /* First time around, look up by name */
    glx_xs_int_closure = caml_named_value("glx_xs_int");
  }
  caml_result = callback_exn(*glx_xs_int_closure, Val_int(i));

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}

int 
glx_xs_integer(int i, item *itp) 
{
  CAMLparam0();  /* This function takes no Caml value arguments */
  CAMLlocal1(caml_result);
  
  static value * glx_xs_integer_closure = NULL;
  if (glx_xs_integer_closure == NULL) {
    /* First time around, look up by name */
    glx_xs_integer_closure = caml_named_value("glx_xs_integer");
  }	
  caml_result = callback_exn(*glx_xs_integer_closure, Val_int(i));

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}

int
glx_xs_decimal(int i, item *itp) 
{
  CAMLparam0();  /* This function takes no Caml value arguments */
  CAMLlocal1(caml_result);
  
  static value * glx_xs_decimal_closure = NULL;
  if (glx_xs_decimal_closure == NULL) {
    /* First time around, look up by name */
    glx_xs_decimal_closure = caml_named_value("glx_xs_decimal");
  }
  caml_result = callback_exn(*glx_xs_decimal_closure, Val_int(i));

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}

int
glx_xs_float(double f, item *itp)
{
  CAMLparam0();  /* This function takes no Caml value arguments */
  CAMLlocal1(caml_result);

  static value * glx_xs_float_closure = NULL;
  if (glx_xs_float_closure == NULL) {
    /* First time around, look up by name */
    glx_xs_float_closure = caml_named_value("glx_xs_float");
  }
  caml_result = callback_exn(*glx_xs_float_closure, copy_double(f));

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}

int
glx_xs_double(double f, item *itp)
{
  CAMLparam0();  /* This function takes no Caml value arguments */
  CAMLlocal1(caml_result);

  item i; 

  static value * glx_xs_double_closure = NULL;
  if (glx_xs_double_closure == NULL) {
    /* First time around, look up by name */
    glx_xs_double_closure = caml_named_value("glx_xs_double");
  }
  caml_result = callback_exn(*glx_xs_double_closure, copy_double(f));

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}

int
glx_xs_anyURI(char* temp, item *itp)
{
  CAMLparam0();  /* This function takes no Caml value arguments */
  CAMLlocal1(caml_result);

  static value * glx_xs_anyURI_closure = NULL;
  if (glx_xs_anyURI_closure == NULL) {
    /* First time around, look up by name */
    glx_xs_anyURI_closure = caml_named_value("glx_xs_anyURI");
  }	
  caml_result = callback_exn(*glx_xs_anyURI_closure, copy_string(temp));

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}

int
glx_xs_QName(module_context c,char *qname, item *itp)
{
  CAMLparam0();  /* This function takes no Caml value arguments */
  CAMLlocal2(args, caml_result);

  item i;

  static value * glx_xs_QName_closure = NULL;
  if (glx_xs_QName_closure == NULL) {
    /* First time around, look up by name */
    glx_xs_QName_closure = caml_named_value("glx_xs_QName");
  }

#ifdef DEBUG
  fprintf (stderr, "Building parameters\n");
#endif

  /* xs_QName takes one tuple argument that is a pair */
  args = alloc_tuple(2); 
  Store_field(args, 0, *c);
  Store_field(args, 1, copy_string(qname));

#ifdef DEBUG
  fprintf (stderr, "callback now...\n");
#endif

  caml_result = callback_exn(*glx_xs_QName_closure, args);

#ifdef DEBUG
  fprintf (stderr, "callback done...\n");
#endif

  if (Is_exception_result(caml_result)) {
#ifdef DEBUG
  fprintf (stderr, "error raised...\n");
#endif

    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}

/*********************/
/* Node constructors */
/*********************/
int
glx_document(char *uri, itemlist kids, item *itp)
{
  CAMLparam0();
  CAMLlocal3(caml_result,caml_kids,args);

  static value * glx_document_closure = NULL;

  if (glx_document_closure == NULL) {
    glx_document_closure = caml_named_value("glx_document");
  }

  caml_kids = camllist_from_itemlist(kids); 

  /* document takes one tuple argument that is a triple */
  args = alloc_tuple(2); 
  Store_field(args, 0, copy_string(uri));
  Store_field(args, 1, caml_kids);
  caml_result = callback_exn(*glx_document_closure, args);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }

}

int
glx_element(item qnameit, itemlist attrs, itemlist kids, item ty_qnameit, item *itp)
{
  CAMLparam0();
  CAMLlocal4(caml_result,caml_attrs,caml_kids,args);

  static value * glx_element_closure = NULL;

  if (glx_element_closure == NULL) {
    glx_element_closure = caml_named_value("glx_element");
  }

  caml_attrs = camllist_from_itemlist(attrs); 
  caml_kids = camllist_from_itemlist(kids); 

  /* element takes one tuple argument that is a quad */
  args = alloc_tuple(4); 
  Store_field(args, 0, *qnameit);
  Store_field(args, 1, caml_attrs);
  Store_field(args, 2, caml_kids);
  Store_field(args, 3, *ty_qnameit);

  caml_result = callback_exn(*glx_element_closure, args);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }

}

int
glx_text(item stringit, item *itp)
{
  CAMLparam0();
  CAMLlocal2(caml_result, string);

  static value * glx_text_closure = NULL;

  if (glx_text_closure == NULL) {
    glx_text_closure = caml_named_value("glx_text");
  }
  string = *stringit;
  caml_result = callback_exn(*glx_text_closure, string);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}
int 
glx_attribute(item qnameit, item stringit, item qnameit1, item *itp)
{
  CAMLparam0();
  CAMLlocal2(caml_result,args); 

  static value * glx_attribute_closure = NULL;

  if (glx_attribute_closure == NULL) {
    glx_attribute_closure = caml_named_value("glx_attribute");

  }

  /* attribute takes one tuple argument that is a triple */
  args = alloc_tuple(3); 
  Store_field(args, 0, *qnameit);
  Store_field(args, 1, *stringit);
  Store_field(args, 2, *qnameit1);
  caml_result = callback_exn(*glx_attribute_closure, args);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }

}
int
glx_comment(item stringit, item *itp)
{
  CAMLparam0();
  CAMLlocal2(caml_result, string);

  static value * glx_comment_closure = NULL;

  if (glx_comment_closure == NULL) {
    glx_comment_closure = caml_named_value("glx_comment");
  }
  string = *stringit;
  caml_result = callback_exn(*glx_comment_closure, string);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}
int
glx_processing_instruction(item stringit,item stringit1, item *itp)
{
  CAMLparam0();
  CAMLlocal4(caml_result,string,string1,args);

  static value * glx_processing_instruction_closure = NULL;

  if (glx_processing_instruction_closure == NULL) {
    glx_processing_instruction_closure = caml_named_value("glx_processing_instruction");
  }
  string = *stringit;
  string1 = *stringit1;

  /* processing_instruction takes one tuple argument that is a pair */
  args = alloc_tuple(2); 
  Store_field(args, 0, string);
  Store_field(args, 1, string1);

  caml_result = callback_exn(*glx_processing_instruction_closure, args);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}


/**********************/
/* Accessors on items */
/**********************/

int
glx_string(item i, char **strp) { 
  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_string_closure = NULL;

  if (glx_string_closure == NULL) {
    glx_string_closure = caml_named_value("glx_string");
  }
  caml_result = callback_exn(*glx_string_closure, *i);
  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *strp = strdup(String_val(caml_result));
    glx_error_string = (char *)NULL;
    CAMLreturn(0);
  }
}

int
glx_item_kind(item i, char **strp)
{
  CAMLparam0();
  CAMLlocal2(caml_result, val);

  static value * glx_item_kind_closure = NULL;
  val = *i;

  if (glx_item_kind_closure == NULL) {
    glx_item_kind_closure = caml_named_value("glx_item_kind");
  }
  caml_result = callback_exn(*glx_item_kind_closure, val);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *strp = strdup(String_val(caml_result));
    glx_error_string = (char *)NULL;
    CAMLreturn(0);
  }
}

int
glx_get_node(item i, node *itp)
{
  CAMLparam0();
  CAMLlocal2(caml_result, val);

  static value * glx_get_node_closure = NULL;
  val = *i;

  if (glx_get_node_closure == NULL) {
    glx_get_node_closure = caml_named_value("glx_get_node");
  }
  caml_result = callback_exn(*glx_get_node_closure, val);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}

int
glx_get_atomicvalue(item i, atomicvalue *itp)
{
  CAMLparam0();
  CAMLlocal2(caml_result, val);

  static value * glx_get_atomicvalue_closure = NULL;
  val = *i;

  if (glx_get_atomicvalue_closure == NULL) {
    glx_get_atomicvalue_closure = caml_named_value("glx_get_atomicvalue");
  }
  caml_result = callback_exn(*glx_get_atomicvalue_closure, val);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}

int 
glx_get_element(item i, element *itp)
{
  CAMLparam0();
  CAMLlocal2(caml_result, val);

  static value * glx_get_element_closure = NULL;
  val = *i; 
  if (glx_get_element_closure == NULL) {
    glx_get_element_closure = caml_named_value("glx_get_element");
  }
  caml_result = callback_exn(*glx_get_element_closure, val);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}

int 
glx_get_attribute(item i, attribute *itp)
{
  CAMLparam0();
  CAMLlocal2(caml_result, val);

  static value * glx_get_attribute_closure = NULL;
  val = *i;

  if (glx_get_attribute_closure == NULL) {
    glx_get_attribute_closure = caml_named_value("glx_get_attribute");
  }
  caml_result = callback_exn(*glx_get_attribute_closure, val);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}

/**********************/
/* Accessors on nodes */
/**********************/
int
glx_parent(node i, itemlist *ilp) 
{
  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_parent_closure = NULL;

  if (glx_parent_closure == NULL) {
    glx_parent_closure = caml_named_value("glx_parent");
  }
  caml_result = callback_exn(*glx_parent_closure, *i);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *ilp = itemlist_from_camllist(caml_result);
    glx_error_string = (char *)NULL;
    CAMLreturn(0);
  }
}

int
glx_children(node i, itemlist *ilp) 
{
  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_children_closure = NULL;

  if (glx_children_closure == NULL) {
    glx_children_closure = caml_named_value("glx_children");
  }
  caml_result = callback_exn(*glx_children_closure, *i);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *ilp = itemlist_from_camllist(caml_result);
    glx_error_string = (char *)NULL;
    CAMLreturn(0);
  }
}

int
glx_node_kind(node i, char **strp) { 
  CAMLparam0();
  CAMLlocal1(caml_result);

  item it; 

  static value * glx_node_kind_closure = NULL;

  if (glx_node_kind_closure == NULL) {
    glx_node_kind_closure = caml_named_value("glx_node_kind");
  }
  caml_result = callback_exn(*glx_node_kind_closure, *i);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *strp = strdup(String_val(caml_result));
    glx_error_string = (char *)NULL;
    CAMLreturn(0);
  }
}

int
glx_typed_value(node i, itemlist *ilp) 
{
  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_typed_value_closure = NULL;

  if (glx_typed_value_closure == NULL) {
    glx_typed_value_closure = caml_named_value("glx_typed_value");
  }
  caml_result = callback_exn(*glx_typed_value_closure, *i);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *ilp = itemlist_from_camllist(caml_result);
    glx_error_string = (char *)NULL;
    CAMLreturn(0);
  }
}

int
glx_attributes(node i, itemlist *ilp) 
{
  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_attributes_closure = NULL;

  if (glx_attributes_closure == NULL) {
    glx_attributes_closure = caml_named_value("glx_attributes");
  }
  caml_result = callback_exn(*glx_attributes_closure, *i);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *ilp = itemlist_from_camllist(caml_result);
    glx_error_string = (char *)NULL;
    CAMLreturn(0);
  }
}

int
glx_base_uri(node i, atomicvalue_list *ilp) 
{
  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_base_uri_closure = NULL;

  if (glx_base_uri_closure == NULL) {
    glx_base_uri_closure = caml_named_value("glx_base_uri");
  }
  caml_result = callback_exn(*glx_base_uri_closure, *i);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *ilp = itemlist_from_camllist(caml_result);
    glx_error_string = (char *)NULL;
    CAMLreturn(0);
  }
}

int
glx_node_name(node i, atomicvalue_list *ilp) 
{
  CAMLparam0();
  CAMLlocal1(caml_result);

  item it; 

  static value * glx_node_name_closure = NULL;

  if (glx_node_name_closure == NULL) {
    glx_node_name_closure = caml_named_value("glx_node_name");
  }
  caml_result = callback_exn(*glx_node_name_closure, *i);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *ilp = itemlist_from_camllist(caml_result);
    glx_error_string = (char *)NULL;
    CAMLreturn(0);
  }

}

/************************/
/* Conversion functions */
/************************/

int
glx_string_of_atomicvalue(atomicvalue a, char **strp) { 
  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_str_of_atomic_closure = NULL;

  if (glx_str_of_atomic_closure == NULL) {
    glx_str_of_atomic_closure = caml_named_value("glx_string_of_atomicvalue");
  }

  caml_result = callback_exn(*glx_str_of_atomic_closure, *a);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *strp = strdup(String_val(caml_result));
    glx_error_string = (char *)NULL;
    CAMLreturn(0);
  }
}

int 
glx_boolean_of_xs_boolean(xs_boolean b, int *intp){ 
  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_bool_of_xs_bool_closure = NULL;

  if (glx_bool_of_xs_bool_closure == NULL) {
    glx_bool_of_xs_bool_closure = caml_named_value("glx_boolean_of_xs_boolean");
  }

  caml_result = callback_exn(*glx_bool_of_xs_bool_closure, *b);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *intp = (Int_val(caml_result));
    glx_error_string = (char *)NULL;
    CAMLreturn(0);
  }
}

int 
glx_int_of_xs_int(xs_int i, int *intp){ 
  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_int_of_xs_int_closure = NULL;

  if (glx_int_of_xs_int_closure == NULL) {
    glx_int_of_xs_int_closure = caml_named_value("glx_int_of_xs_int");
  }
  caml_result = callback_exn(*glx_int_of_xs_int_closure, *i);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *intp = (Int_val(caml_result));
    glx_error_string = (char *)NULL;
    CAMLreturn(0);
  }
}

int 
glx_integer_of_xs_integer(xs_integer i, int *intp){ 
  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_integer_of_xs_integer_closure = NULL;

  if (glx_integer_of_xs_integer_closure == NULL) {
    glx_integer_of_xs_integer_closure = caml_named_value("glx_integer_of_xs_integer");
  }
  caml_result = callback_exn(*glx_integer_of_xs_integer_closure, *i);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *intp = (Int_val(caml_result));
    glx_error_string = (char *)NULL;
    CAMLreturn(0);
  }
}

int 
glx_decimal_of_xs_decimal(xs_decimal i, int *intp){ 
  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_decimal_of_xs_decimal_closure = NULL;

  if (glx_decimal_of_xs_decimal_closure == NULL) {
    glx_decimal_of_xs_decimal_closure = caml_named_value("glx_decimal_of_xs_decimal");
  }
  caml_result = callback_exn(*glx_decimal_of_xs_decimal_closure, *i);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *intp = (Int_val(caml_result));
    glx_error_string = (char *)NULL;
    CAMLreturn(0);
  }
}

int
glx_float_of_xs_float(xs_float i, double *dbp){ 
  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_float_of_xs_float_closure = NULL;

  if (glx_float_of_xs_float_closure == NULL) {
    glx_float_of_xs_float_closure = caml_named_value("glx_float_of_xs_float");
  }
  caml_result = callback_exn(*glx_float_of_xs_float_closure, *i);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *dbp = (Double_val(caml_result));
    glx_error_string = (char *)NULL;
    CAMLreturn(0);
  }
}

int
glx_float_of_xs_double(xs_double d, double *dbp){ 
  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_float_of_xs_double_closure = NULL;

  if (glx_float_of_xs_double_closure == NULL) {
    glx_float_of_xs_double_closure = caml_named_value("glx_float_of_xs_double");
  }
  caml_result = callback_exn(*glx_float_of_xs_double_closure, *d);

 if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *dbp = (Double_val(caml_result));
    glx_error_string = (char *)NULL;
    CAMLreturn(0);
  }
}


/*********************************/
/* Document I/O functions        */
/*********************************/
int
glx_load_document(char* xml, itemlist *ilp)
{
  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_load_document_closure = NULL;

  if (glx_load_document_closure == NULL) {
    glx_load_document_closure = caml_named_value("glx_load_document");
  }
  caml_result = callback_exn(*glx_load_document_closure, copy_string(xml));

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *ilp = itemlist_from_camllist(caml_result);
    glx_error_string = "";
    CAMLreturn(0);
  }
}

int
glx_load_document_validate(item context, char* xml, itemlist *ilp)
{
  CAMLparam0();
  CAMLlocal2(caml_result, context1);

  static value * glx_load_document_validate_closure = NULL;
  context1 = *context;

  if (glx_load_document_validate_closure == NULL) {
    glx_load_document_validate_closure = caml_named_value("glx_load_document_validate");
  }
  caml_result = callback2_exn(*glx_load_document_validate_closure, context1, copy_string(xml));

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *ilp = itemlist_from_camllist(caml_result);
    glx_error_string = "";
    CAMLreturn(0);
  }
}

int
glx_serialize_to_string(itemlist items, char **strp)
{
  CAMLparam0();
  CAMLlocal2(camllist, caml_result);

  static value * glx_serialize_to_string_closure = NULL;

  if (glx_serialize_to_string_closure == NULL) {
    glx_serialize_to_string_closure = caml_named_value("glx_serialize_to_string");
  }

  camllist = camllist_from_itemlist(items);

  caml_result = callback_exn(*glx_serialize_to_string_closure, camllist);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *strp = strdup(String_val(caml_result));
    glx_error_string = (char *)NULL;
    CAMLreturn(0);
  }
}

int
glx_serialize_to_output_channel(itemlist il) 
{
  CAMLparam0();
  CAMLlocal2(camllist, caml_result);

  static value * glx_serialize_to_output_channel_closure = NULL;

  camllist = camllist_from_itemlist(il);

  if (glx_serialize_to_output_channel_closure == NULL) {
    glx_serialize_to_output_channel_closure = caml_named_value("glx_serialize_to_output_channel");
  }
  caml_result = callback_exn(*glx_serialize_to_output_channel_closure,camllist); 
   
  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    glx_error_string = (char *)NULL;
    CAMLreturn(0);
  }
}

int
glx_serialize_to_file(char *out, itemlist il)
{
  CAMLparam0();
  CAMLlocal2(camllist, caml_result);

  static value * glx_serialize_to_file_closure = NULL;

  camllist = camllist_from_itemlist(il);

  if (glx_serialize_to_file_closure == NULL) {
    glx_serialize_to_file_closure = caml_named_value("glx_serialize_to_file");
  }
  caml_result = callback2_exn(*glx_serialize_to_file_closure, copy_string(out), camllist);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    glx_error_string = (char *)NULL;
    CAMLreturn(0);
  }
}
 
/*********************************/
/* Evaluation context            */
/*********************************/

int
glx_default_processing_context(processing_context *itp)
{
  CAMLparam0();
  CAMLlocal1(caml_result);
  int err;

  static value * glx_default_context_closure = NULL;

  if (glx_default_context_closure == NULL) {
    glx_default_context_closure = caml_named_value("glx_default_processing_context");
  }

  caml_result = callback_exn(*glx_default_context_closure, Val_int(0));

  if (Is_exception_result(caml_result)) {
    *itp = NULL;
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    err = -1;
  } else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    err = 0;
  }

  CAMLreturn(err);
}

int 
glx_load_standard_library(processing_context pc, module_context *itp)
{
  CAMLparam0();
  CAMLlocal1(caml_result);
  int err;

  static value * glx_build_context_from_file_closure = NULL;

  if (glx_build_context_from_file_closure == NULL) {
    glx_build_context_from_file_closure = caml_named_value("glx_load_standard_library");
  }

  caml_result = callback_exn(*glx_build_context_from_file_closure, *pc);

  if (Is_exception_result(caml_result)) {
    *itp = NULL;
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    err = -1;
  } else {
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    err = 0; 
  }

  CAMLreturn(err);
}

int 
glx_import_library_module(module_context con, char *filename, module_context *itp)
{
  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_build_context_from_string_closure = NULL;

  if (glx_build_context_from_string_closure == NULL) {
    glx_build_context_from_string_closure = caml_named_value("glx_import_library_module");
  }
  caml_result = callback2_exn(*glx_build_context_from_string_closure, *con, copy_string(filename));

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}

int 
glx_import_main_module(module_context con, char *filename, query_context *itp)
{
  CAMLparam0();
  CAMLlocal2(caml_result,caml_attrs);

  static value * glx_import_main_module_closure = NULL;

  if (glx_import_main_module_closure == NULL) {
    glx_import_main_module_closure = caml_named_value("glx_import_main_module");
  }
  caml_result = callback2_exn(*glx_import_main_module_closure,*con, copy_string(filename));

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}

int
glx_import_main_module_from_string(module_context con, char *str, query_context *itp)
{
  CAMLparam0();
  CAMLlocal2(caml_result,caml_attrs);

  module_context con1;

  static value * glx_import_main_module_closure = NULL;

  if (glx_import_main_module_closure == NULL) {
    glx_import_main_module_closure = caml_named_value("glx_import_main_module_from_string");
  }
  caml_result = callback2_exn(*glx_import_main_module_closure,*con, copy_string(str));

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}


int  
glx_build_external_context(itemlist ctxt_item, char **varnames, itemlist values[], int nargs, external_context *itp) {
  CAMLparam0();
  CAMLlocal4(caml_result, ctxt_item_list, var_list, val_list);

  static value * glx_build_external_context = NULL;

  if (glx_build_external_context == NULL) {
    glx_build_external_context = caml_named_value("glx_build_external_context");
  }

  ctxt_item_list = camllist_from_itemlist(ctxt_item); 
  var_list = camlstrings_from_strings(varnames, nargs); 
  val_list = camllistlist_from_itemlistlist(values, nargs); 

  caml_result = callback3_exn(*glx_build_external_context, ctxt_item_list, var_list, val_list);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }

}

int
glx_add_external_context(module_context mod_ctxt, external_context ext_ctxt, module_context  *itp) { 
  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_add_external_context = NULL;

  if (glx_add_external_context == NULL) {
    glx_add_external_context = caml_named_value("glx_add_external_context");
  }
  caml_result = callback2_exn(*glx_add_external_context, *mod_ctxt, *ext_ctxt);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}

int 
glx_eval_global_variables(module_context con, module_context *itp)
{
  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_eval_global_vars_closure = NULL;

  if (glx_eval_global_vars_closure == NULL) {
    glx_eval_global_vars_closure = caml_named_value("glx_eval_global_variables");
  }

  caml_result = callback_exn(*glx_eval_global_vars_closure, *con);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *itp = (item) malloc(sizeof(value));
    register_global_root(*itp);  
    **itp = caml_result;
    glx_error_string = (char *)NULL;
    CAMLreturn(0); 
  }
}	

/*********************************/
/* Query evaluation              */
/*********************************/

int
glx_eval_statement_from_string(module_context con, char* querystr, itemlist *ilp)
{
  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_eval_statement_from_string_closure = NULL;

  if (glx_eval_statement_from_string_closure == NULL) {
    glx_eval_statement_from_string_closure = caml_named_value("glx_eval_statement_from_string");
  }

  caml_result = callback2_exn(*glx_eval_statement_from_string_closure, *con, copy_string(querystr));

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *ilp = itemlist_from_camllist(caml_result);
    glx_error_string = "";
    CAMLreturn(0);
  }
}

int
glx_eval_statements(query_context con, itemlist *ilp)
{
  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_eval_statements_closure = NULL;

  if (glx_eval_statements_closure == NULL) {
    glx_eval_statements_closure = caml_named_value("glx_eval_statements");
  }

  caml_result = callback_exn(*glx_eval_statements_closure, *con);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *ilp = itemlist_from_camllist(caml_result);
    glx_error_string = "";
    CAMLreturn(0);
  }
}	

int
glx_eval_query_function(module_context con, char* funname, itemlist *args, int nargs, itemlist *ilp)
{
  CAMLparam0();
  CAMLlocal4(caml_items,caml_list, caml_result, last);

  itemlist il; 
  int argct; 

  static value * glx_eval_query_fun_closure = NULL;

  if (glx_eval_query_fun_closure == NULL) {
    glx_eval_query_fun_closure = caml_named_value("glx_eval_query_function");
  }	

  caml_list = camllistlist_from_itemlistlist(args, nargs); 
  caml_result = callback3_exn(*glx_eval_query_fun_closure, *con, copy_string(funname), caml_list);

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *ilp = itemlist_from_camllist(caml_result);
    glx_error_string = "";
    CAMLreturn(0);
  }
}

int
glx_eval_statement_with_context_item_from_document(char *file, char *querystr, itemlist *ilp) 
{
  CAMLparam0();
  CAMLlocal1(caml_result);
  int err;

  static value * glx_eval_statement_with_context_item_from_document_closure = NULL;

#ifdef DEBUG
  fprintf(stderr,"Query String : %s\n",querystr);
#endif

  if (glx_eval_statement_with_context_item_from_document_closure == NULL) {
    glx_eval_statement_with_context_item_from_document_closure = caml_named_value("glx_eval_statement_with_context_item_from_document");
  }

#ifdef DEBUG
  fprintf(stderr,"Performing the Caml callback\n",querystr);
#endif

  caml_result = callback2_exn(*glx_eval_statement_with_context_item_from_document_closure, copy_string(file), copy_string(querystr));

#ifdef DEBUG
  fprintf(stderr,"Caml callback done\n",querystr);
#endif

  if (Is_exception_result(caml_result)) {
#ifdef DEBUG
    fprintf(stderr,"An exception was raised\n",querystr);
#endif

    *ilp = itemlist_empty();
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 

#ifdef DEBUG
    fprintf(stderr,"Returning from exception conversion with message: %s\n",glx_error_string);
#endif

    err = -1;
  } else { 
    *ilp = itemlist_from_camllist(caml_result);
    glx_error_string = "";
    err = 0;
  }

  CAMLreturn(err);
}

int
glx_eval_statement_with_context_item_from_xml(item ctxt_item, char *querystr, itemlist *ilp) 
{

  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_eval_statement_with_context_item_from_xml_closure = NULL;

#ifdef DEBUG
  fprintf(stderr,"Query String : %s\n",querystr);
#endif

  if (glx_eval_statement_with_context_item_from_xml_closure == NULL) {
    glx_eval_statement_with_context_item_from_xml_closure = caml_named_value("glx_eval_statement_with_context_item_from_xml");
  }

  caml_result = callback2_exn(*glx_eval_statement_with_context_item_from_xml_closure, *ctxt_item, copy_string(querystr));

#ifdef DEBUG
  fprintf(stderr,"Query was evaluated\n");
#endif

  if (Is_exception_result(caml_result)) {
#ifdef DEBUG
  fprintf(stderr,"Query raised an error\n");
#endif

  glx_error_string = glx_exception_string(Extract_exception(caml_result)); 

#ifdef DEBUG
  fprintf(stderr,"Error message : %s\n",glx_error_string);
#endif

    CAMLreturn(-1);
  }
  else { 
    *ilp = itemlist_from_camllist(caml_result);
    glx_error_string = "";
    CAMLreturn(0);
  }
}

int
glx_eval_statement_from_file_with_context_item_from_xml(item ctxt_item, char *queryfile, itemlist *ilp) 
{
  CAMLparam0();
  CAMLlocal1(caml_result);

  static value * glx_eval_statement_from_file_with_context_item_from_xml_closure = NULL;

#ifdef DEBUG
  fprintf(stderr,"Query file : %s\n",queryfile);
#endif

  if (glx_eval_statement_from_file_with_context_item_from_xml_closure == NULL) {
    glx_eval_statement_from_file_with_context_item_from_xml_closure = caml_named_value("glx_eval_statement_from_file_with_context_item_from_xml");
  }

  caml_result = callback2_exn(*glx_eval_statement_from_file_with_context_item_from_xml_closure, *ctxt_item, copy_string(queryfile));

  if (Is_exception_result(caml_result)) {
    glx_error_string = glx_exception_string(Extract_exception(caml_result)); 
    CAMLreturn(-1);
  }
  else { 
    *ilp = itemlist_from_camllist(caml_result);
    glx_error_string = "";
    CAMLreturn(0);
  }
}

int
glx_eval_statement_with_variables_from_xml(char **varnames, itemlist *values, int nargs, char *querystr, itemlist *ilp) 
{
  CAMLparam0();
  CAMLlocal3(caml_result, var_list,val_list);
  int err;

  static value * glx_eval_statement_with_variables_from_xml_closure = NULL;

  if (glx_eval_statement_with_variables_from_xml_closure == NULL) {
    glx_eval_statement_with_variables_from_xml_closure = caml_named_value("glx_eval_statement_with_variables_from_xml");
  }
  var_list = camlstrings_from_strings(varnames, nargs); 
  val_list = camllistlist_from_itemlistlist(values, nargs); 

  caml_result = callback3_exn(*glx_eval_statement_with_variables_from_xml_closure, var_list, val_list, copy_string(querystr));

  if (Is_exception_result(caml_result)) {
    *ilp = NULL;
    glx_error_string = glx_exception_string(Extract_exception(caml_result));
    err = -1;
  } else {
    *ilp = itemlist_from_camllist(caml_result);
    glx_error_string = "";
    err = 0;
  }

  CAMLreturn(err);
}
