(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                              XQuery Engine                          *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

open Datamodel_api

(*****************************)
(* Atomic value constructors *)
(*****************************)
val glx_xs_string : Datatypes.xs_string -> xs_string
val glx_xs_boolean : Datatypes.xs_boolean -> xs_boolean

(* Currently, these wrapper functions all take native int *)
val glx_xs_int :     Datatypes.xs_int -> xs_int
val glx_xs_integer : Datatypes.xs_int -> xs_integer
val glx_xs_decimal : Datatypes.xs_int -> xs_decimal

(* Currently, these wrapper functions all take native float *)
val glx_xs_float : Datatypes.xs_float -> xs_float
val glx_xs_double : Datatypes.xs_float -> xs_double
val glx_xs_anyURI : Datatypes.xs_string -> xs_anyURI
val glx_xs_untyped : string -> xs_untyped
val glx_xs_QName : Galapi.module_context * string -> xs_QName

(*********************)
(* Node constructors *)
(*********************)
val glx_document :
  (string * node list) -> document
val glx_element :
  xs_QName * attribute list *
  node list * xs_QName -> element
val glx_attribute :
  xs_QName * xs_string * xs_QName ->
  attribute
val glx_text : xs_string -> text
val glx_comment : xs_string -> comment
val glx_processing_instruction :
  xs_string * xs_string ->
  processing_instruction

(**********************)
(* Accessors on nodes *)
(**********************)
val glx_string : item -> string
val glx_item_kind : item -> string
val glx_get_node : item -> node
val glx_get_atomicvalue : item -> atomicvalue
val glx_get_element : item -> element
val glx_get_attribute : item -> attribute

val glx_parent : node -> node list
val glx_children : node -> node list

(* Note: glx_base_uri returns empty list or singleton list containing xs_anyURI value *)
val glx_base_uri : node -> xs_anyURI list
val glx_node_kind : node -> string
(* Note: glx_node_name returns empty list or singleton list containing xs_string value *)
val glx_node_name : node -> xs_QName list
val glx_typed_value : node -> item list
val glx_attributes : node -> node list
val glx_namespace : node -> node list

(************************) 
(* Conversion functions *)
(************************) 

val glx_string_of_atomicvalue  : atomicvalue -> Datatypes.xs_string 
val glx_boolean_of_xs_boolean  : xs_boolean -> Datatypes.xs_boolean
val glx_int_of_xs_int          : xs_int -> Datatypes.xs_int
val glx_integer_of_xs_integer  : xs_integer -> Datatypes.xs_int
val glx_decimal_of_xs_decimal  : xs_decimal -> Datatypes.xs_int
val glx_float_of_xs_float      : xs_float -> Datatypes.xs_float
val glx_float_of_xs_double     : xs_double -> Datatypes.xs_float

(**************************)
(* Document I/O functions *)
(**************************)
val glx_load_document : string -> node list
val glx_load_document_validate :  Galapi.module_context -> string -> node list
val glx_serialize_to_string : item list -> string
val glx_serialize_to_file : string -> item list -> unit
val glx_serialize_to_output_channel : item list -> unit

(**********************)
(* Evaluation context *)
(**********************)

(* Default processing context *)
val glx_default_processing_context : unit -> Galapi.processing_context

(* Compile a query prolog : includes normalization, typing, & optimization *)
val glx_load_standard_library : Galapi.processing_context -> Galapi.module_context

val glx_import_library_module : Galapi.module_context -> string -> Galapi.module_context
val glx_import_main_module    : Galapi.module_context -> string -> Galapi.module_context * (Galapi.xquery_statement list)
val glx_import_main_module_from_string : Galapi.module_context -> string -> Galapi.module_context * (Galapi.xquery_statement list)

(* 
   External context: 
   Context item 
   External variables and their values
   External functions 

   Redefining any symbol raises an error
*)

val glx_build_external_context : (Datamodel_api.item list) -> string list -> (Datamodel_api.item list) list -> Galapi.external_context
val glx_add_external_context : Galapi.module_context -> Galapi.external_context -> Galapi.module_context
val glx_eval_global_variables : Galapi.module_context -> Galapi.module_context (* different type? *)

(********************)
(* Query evaluation *)
(********************)

(* Evaluate *)

val glx_eval_statement_from_string : Galapi.module_context -> string -> item list 
val glx_eval_query_function  : Galapi.module_context -> string -> item list list -> item list
val glx_eval_statements : (Galapi.module_context * Galapi.xquery_statement list) -> item list 

(* Quick start evaluation functions *)

val glx_eval_statement_with_context_item_from_document : string -> string -> item list 
val glx_eval_statement_with_context_item_from_xml : item -> string -> item list 
val glx_eval_statement_from_file_with_context_item_from_xml : item -> string -> item list 
val glx_eval_statement_with_variables_from_xml : string list -> item list list -> string -> item list


