/***********************************************************************/
/*                                                                     */
/*                                 GALAX                               */
/*                              XQuery Engine                          */
/*                                                                     */
/*  Copyright 2001-2003, Lucent Technologies.                          */
/*  Distributed only by permission.                                    */
/*                                                                     */
/***********************************************************************/


/* $Id: glx_jni_stub.c,v 1.8 2003/06/30 19:46:45 simeon Exp $ */

#include <string.h>

#include <caml/mlvalues.h>
#include <caml/callback.h>
#include <caml/memory.h>

#include <jni.h>

#include "glx.h"

#include "galapi_Galax.h"
#include "galapi_Item.h"
#include "galapi_ItemList.h"
#include "galapi_Atomic.h"
#include "galapi_xsString.h"
#include "galapi_xsBoolean.h"
#include "galapi_xsInt.h"
#include "galapi_xsInteger.h"
#include "galapi_xsDecimal.h"
#include "galapi_xsFloat.h"
#include "galapi_xsDouble.h"
#include "galapi_xsAnyURI.h"
#include "galapi_xsUntyped.h"
#include "galapi_xsQName.h"
#include "galapi_Node.h"
#include "galapi_Document.h"
#include "galapi_Element.h"
#include "galapi_Text.h"
#include "galapi_Attribute.h"
#include "galapi_Comment.h"
#include "galapi_ProcessingInstruction.h"
#include "galapi_ProcessingContext.h"
#include "galapi_ExternalContext.h"
#include "galapi_QueryContext.h"
#include "galapi_ModuleContext.h"
#include "galapi_NodeList.h"
#include "galapi_AttributeList.h"
#include "galapi_AtomicList.h"

/* #define DEBUG */

/* Exception handling */

void throw_exception (JNIEnv *env, char *exception, 
		      char *message,
		      int code) {
  jclass newExcCls;

  // (*env)->ExceptionDescribe(env);
  // (*env)->ExceptionClear(env);

  newExcCls = (*env)->FindClass(env, exception);
  if (newExcCls == NULL) {
    fprintf (stderr, "Unable to load the exception class %s\n", exception);
    fflush (stderr);
    return;
  }
#ifdef DEBUG
  fprintf (stderr, "Exception class %s found!\n", exception);
#endif

  (*env)->ThrowNew(env, newExcCls, message);
}

void
throw_galapi_exception (JNIEnv *env, char *message, int err) {
#ifdef DEBUG
  fprintf(stderr,"Throwing Java exception with message: %s\n", message);
#endif

  //  why doesn't this work ??? :
  throw_exception (env, "galapi/GalapiException", message, err);
  // throw_exception (env, "java/lang/Exception", message, err);
}


/**
 * Class:     Galax
 * Method:    initialize
 * @param jne
 * @param jo
 * @param jarr array holding the program arguments
 */



JNIEXPORT void JNICALL
 Java_galapi_Galax_nativeInitialize (JNIEnv * jne, jobject jo)
{
  
#ifdef DEBUG
  fprintf (stderr, "Calling glx_init() from Java\n");
#endif

  glx_init ();

#ifdef DEBUG
  fprintf (stderr, "Successful glx_init() called from Java stub\n");
#endif

}// Java_GALAX_initialize()



/*
 * jarr is an array of java.lang.String
 * @return a C array of (char*)
 */

char ** getStringArray (JNIEnv * jne, jobjectArray jarr) {

  jsize sz = (*jne)->GetArrayLength (jne, jarr);
  char **a = (char **)malloc (sz * sizeof(char *));
  jsize i;
  const char *s;
  jstring js;

  for (i=0; i<sz; i++) {
    js = (jstring) (*jne)->GetObjectArrayElement (jne, jarr, i);
    s = (*jne)->GetStringUTFChars (jne, js, NULL); 
    a[i] = (char *) s;
  }//end for

  return a;
}


void releaseStringArray (JNIEnv * jne, jobjectArray jarr, char **a) {

  jsize sz = (*jne)->GetArrayLength (jne, jarr);
  jsize j;
  jstring js;

  for (j=0; j<sz; j++) {
    js = (jstring) (*jne)->GetObjectArrayElement (jne, jarr, j);
    (*jne)->ReleaseStringUTFChars (jne, js, a[j]);
  }//end for
  
  free (a);
}

/*
 * class Context
 */



JNIEXPORT void JNICALL 
Java_galapi_Context_nativeFree (JNIEnv * jne, jclass jc, jint ji) {
  item_free ((item)ji);

}// Java_Context_nativeFree()



/*
 * class Item
 */

JNIEXPORT void JNICALL 
Java_galapi_Item_nativeFree (JNIEnv * jne, jclass jc, jint ji) {
  item_free ((item)ji);

}// Java_Item_nativeFree()


 

JNIEXPORT jint JNICALL
Java_galapi_Item_nativeAtomicValue (JNIEnv *jne, jclass jc, jint ji) {
  atomicvalue v;
  glx_err err = glx_get_atomicvalue ((item) ji, &v);
  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);
  return (jint) v;

}// Java_Item_nativeAtomicValue()

JNIEXPORT jint JNICALL
Java_galapi_Item_nativeNode (JNIEnv *jne, jclass jc, jint ji) {
  element e;
  glx_err err;

#ifdef DEBUG
  fprintf (stderr,"GALAX GET NODE2\n");
#endif
  err = glx_get_node ((item) ji, &e);

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);
    
#ifdef DEBUG
  fprintf (stderr,"OUT OF GALAX GET NODE2\n");
#endif
  return (jint) e;

}// Java_Item_nativeNode()

JNIEXPORT jint JNICALL
Java_galapi_Item_nativeAttribute (JNIEnv *jne, jclass jc, jint ji) {
  element e;
  glx_err err = glx_get_element ((item) ji, &e);
  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) e;

}// Java_Item_nativeAttribute()


JNIEXPORT jint JNICALL
Java_galapi_Item_nativeElement (JNIEnv *jne, jclass jc, jint ji) {
  element e;
  glx_err err = glx_get_attribute ((item) ji, &e);
  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) e;

}// Java_Item_nativeElement()


JNIEXPORT jstring JNICALL
Java_galapi_Item_nativeString (JNIEnv *jne, jclass jc, jint ji) {
  char *s;
  glx_err err = glx_string ((item) ji, &s);
  if (0 != err)
    throw_exception (jne, "java/lang/RuntimeException", glx_error_string, err);
  
  return (*jne)->NewStringUTF(jne, s);
}// Java_Item_nativeString()


JNIEXPORT jstring JNICALL
Java_galapi_Item_nativeItemKind (JNIEnv *jne, jclass jc, jint ji) {
  char *s;
  glx_err err = glx_item_kind ((item) ji, &s);
  if (0 != err)
    throw_exception (jne, "java/lang/RuntimeException", glx_error_string, err);
  
  return (*jne)->NewStringUTF(jne, s);
}// Java_Item_nativeString()



/*
 * class ItemList
 */


JNIEXPORT jint JNICALL 
Java_galapi_ItemList_nativeEmptyList (JNIEnv *jne, jclass jc) {
  return (jint) itemlist_empty();  
}// Java_ItemList_nativeEmptyList()


JNIEXPORT void JNICALL 
Java_galapi_ItemList_nativeFreeItemList
(JNIEnv *jne, jclass jc, jint ji) {
  itemlist_free ((itemlist)ji);

}// Java_ItemList_nativeFreeItemList()


JNIEXPORT jint JNICALL
Java_galapi_ItemList_nativeCons (JNIEnv *jne, jclass jc, jint jit, jint jitl) {
  itemlist il = itemlist_cons ((item) jit, (itemlist) jitl);
  return (jint) il;

}// Java_Item_nativeCons()

JNIEXPORT jboolean JNICALL 
Java_galapi_ItemList_nativeIsEmpty (JNIEnv *jne, jclass jc, jint ji) {
  int b = is_empty ((itemlist) ji);
  return (jboolean) b;

}// Java_ItemList_nativeIsEmpty()


JNIEXPORT jstring JNICALL 
Java_galapi_ItemList_nativeSerialize (JNIEnv *jne, jclass jc, jint ji) {
  char *s = NULL;
  glx_err err = glx_serialize_to_string ((itemlist) ji, &s);
  
  if (0 != err)
    throw_exception (jne, "java/lang/RuntimeException", glx_error_string, err);

  return (*jne)->NewStringUTF(jne, s);
}// Java_ItemList_nativeSerialize()



JNIEXPORT jint JNICALL
Java_galapi_ItemList_nativeItemsFirst (JNIEnv *jne, jclass jc, jint jitl) {
  item i = items_first  ((itemlist) jitl);
  return (jint) i;

}// Java_ItemList_nativeItemsFirst


JNIEXPORT jint JNICALL
Java_galapi_ItemList_nativeItemsNext (JNIEnv *jne, jclass jc, jint jitl) {
  itemlist il = items_next  ((itemlist) jitl);
  return (jint) il;

}// Java_Item_nativeIemsNext()

/*
 *
 * clas Atomic
 *
 */

JNIEXPORT jstring JNICALL
Java_galapi_Atomic_nativeSerialize (JNIEnv *jne, jclass jc, jint ji) {  
  char *s;
  glx_err err = glx_string_of_atomicvalue ((atomicvalue) ji, &s);

  if (0 != err)
    throw_exception (jne, "java/lang/RuntimeException", glx_error_string, err);

  return (*jne)->NewStringUTF(jne, s);
}// Java_Atomic_nativeSerialize()


/*
 * class xsString
 */

JNIEXPORT jint JNICALL 
Java_galapi_xsString_stringToItem (JNIEnv *jne, jclass jc, jstring js) {

  const char *str = (*jne)->GetStringUTFChars(jne, js, NULL);
  xs_string it;
  glx_err err = glx_xs_string ((char *)str, &it);

  (*jne)->ReleaseStringUTFChars (jne, js, str);

  // Attention: the exception should always be thrown after having
  // released all the resources

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) it;

}// Java_xsString_stringToItem()


/*
 * xsBoolean
 */

JNIEXPORT jint JNICALL 
Java_galapi_xsBoolean_booleanToItem
(JNIEnv *jne, jclass jc, jboolean jb) {
  xs_boolean xsb;
  glx_err err = glx_xs_boolean ((int) jb, &xsb);

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) xsb;

}// Java_xsBoolean_booleanToItem()


JNIEXPORT jboolean JNICALL Java_galapi_xsBoolean_itemToBoolean
(JNIEnv *jne, jclass jc, jint ji) {
  int b;
  glx_err err = glx_boolean_of_xs_boolean ((xs_boolean) ji, &b);
  
  if (0 != err)
      throw_galapi_exception (jne, glx_error_string, err);

  return (jboolean) b;

}// Java_xsBoolean_itemToBoolean()



/*
 * xsInteger
 */

JNIEXPORT jint JNICALL Java_galapi_xsInteger_integerToItem
(JNIEnv *jne, jclass jc, jint ji){
  xs_integer xi;
  glx_err err = glx_xs_integer ((int) ji, &xi);

  if (0 != err)
      throw_galapi_exception (jne, glx_error_string, err);
  
  return (jint) xi;  

} // Java_xsInteger_integerToItem()


JNIEXPORT jint JNICALL Java_galapi_xsInteger_itemToInteger
(JNIEnv *jne, jclass jc, jint ji) {
  int d;
  
  glx_err err = glx_integer_of_xs_integer ((xs_integer) ji, &d);

  if (0 != err)
      throw_galapi_exception (jne, glx_error_string, err);
  
  return (jint) d;

}// Java_xsInteger_itemToInteger ()


/*
 * xsInt
 */

JNIEXPORT jint JNICALL Java_galapi_xsInt_intToItem
(JNIEnv *jne, jclass jc, jint ji){
  xs_int xi;
  glx_err err = glx_xs_int ((int) ji, &xi);

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) xi;  

} // Java_xsInt_integerToItem()


JNIEXPORT jint JNICALL Java_galapi_xsInt_itemToInt
(JNIEnv *jne, jclass jc, jint ji) {
  int d;
  glx_err err = glx_int_of_xs_int ((xs_int) ji, &d);
  
  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);
  
  return (jint) d;

}// Java_xsInt_itemToInteger ()

/*
 * xsDecimal
 */

JNIEXPORT jint JNICALL Java_galapi_xsDecimal_decimalToItem
(JNIEnv *jne, jclass jc, jint ji){
  xs_decimal xi;
  glx_err err = glx_xs_decimal ((int) ji, &xi);
  
  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);
  
  return (jint) xi;  

} // Java_xsDecimal_decimalToItem()


JNIEXPORT jint JNICALL Java_galapi_xsDecimal_itemToDecimal
(JNIEnv *jne, jclass jc, jint ji) {
  int d;
  glx_err err = glx_decimal_of_xs_decimal ((xs_decimal) ji, &d);

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) d;

}// Java_xsDecimal_itemToDecimal()


/*
 * xsFloat
 */

JNIEXPORT jint JNICALL Java_galapi_xsFloat_floatToItem
(JNIEnv *jne, jclass jc, jdouble jf){
  xs_float xf;
  glx_err err = glx_xs_float ((double) jf, &xf);

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);
  
  return (jint) xf;  

} // Java_xsFloat_floatToItem()


JNIEXPORT jdouble JNICALL Java_galapi_xsFloat_itemToFloat
(JNIEnv *jne, jclass jc, jint ji) {
  double f;
  glx_err err = glx_float_of_xs_float ((xs_float) ji, &f);

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jdouble) f;

}// Java_xsFloat_itemToFloat()



/*
 * xsDouble
 */

JNIEXPORT jint JNICALL Java_galapi_xsDouble_doubleToItem
(JNIEnv *jne, jclass jc, jdouble jf){
  xs_double xg;
  glx_err err = glx_xs_double ((double) jf, &xg);

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) xg;  

}// Java_xsDouble_doubleToItem()


JNIEXPORT jdouble JNICALL Java_galapi_xsDouble_itemToDouble
(JNIEnv *jne, jclass jc, jint ji) {
  double g;
  glx_err err = glx_float_of_xs_double ((xs_double) ji, &g);

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jdouble) g;

}// Java_xsDouble_itemToDouble()


/*
 * class xsAnyURI
 */

JNIEXPORT jint JNICALL 
Java_galapi_xsAnyURI_uriToItem (JNIEnv *jne, jclass jc, jstring js) {


  const char *str = (*jne)->GetStringUTFChars(jne, js, NULL);
  xs_anyURI it;
  glx_err err = glx_xs_anyURI ((char *)str, &it);

  (*jne)->ReleaseStringUTFChars (jne, js, str);

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) it;

}// Java_xsAnyURI_uriToItem()


/*
 * xsQName
 */

JNIEXPORT jint JNICALL Java_galapi_xsQName_qNameToItem
(JNIEnv *jne, jclass jc, jint ji, jstring js) {
  
  const char *name = (*jne)->GetStringUTFChars(jne, js, NULL);  

  xs_QName it;
  glx_err err;

#ifdef DEBUG
  fprintf (stderr, "Calling xsQName\n");
#endif

  err = glx_xs_QName ((module_context) ji, 
		      (char *)name, &it);
  (*jne)->ReleaseStringUTFChars (jne, js, name);

#ifdef DEBUG
  fprintf (stderr, "xsQName call done\n");
#endif

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);
  
  return (jint) it; 

}// Java_xsQName_qNameToItem()


/*
 * Document
 */

JNIEXPORT jint JNICALL Java_galapi_Document_documentToItem
(JNIEnv *jne, jclass jc, jstring juri, jint nodes) {
  
  document d;

  const char *str = (*jne)->GetStringUTFChars(jne, juri, NULL);
  glx_err err = glx_document (str, (itemlist) nodes, &d);
  
  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) d;

}// Java_Element_documentToItem()

/*
 * Element
 */

JNIEXPORT jint JNICALL Java_galapi_Element_elementToItem
(JNIEnv *jne, jclass jc, jint jq, jint attrs, jint nodes, jint jq1) {
  
  element e;
  glx_err err = glx_element ((xs_QName) jq, 
			     (attribute_list) attrs,
			     (itemlist) nodes,
			     (xs_QName) jq1,
			     &e);
  
  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) e;

}// Java_Element_elementToItem()


/*
 * Text
 */

JNIEXPORT jint JNICALL Java_galapi_Text_textToItem
(JNIEnv * jne, jclass jc, jint ji) {
  
  text t;
  glx_err err = glx_text ((xs_string) ji, &t);

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) t;
}// Java_Text_textToItem


/*
 * Attribute
 */

JNIEXPORT jint JNICALL Java_galapi_Attribute_attributeToItem
(JNIEnv *jne, jclass jc, jint q, jint s, jint q1) {
  
  attribute a;
  glx_err err = glx_attribute ((xs_QName)q, (xs_string) s, (xs_QName)q1,
			       &a);

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) a;
}// Java_Attribute_attributeToItem()



/*
 * Comment
 */

JNIEXPORT jint JNICALL Java_galapi_Comment_commentToItem
(JNIEnv * jne, jclass jc, jint ji) {
  
  text t;
  glx_err err = glx_comment ((xs_string) ji, &t);

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) t;
}// Java_Comment_commentToItem

/*
 * ProcessingInstruction
 */


JNIEXPORT jint JNICALL 
Java_galapi_ProcessingInstruction_processingInstructionToItem
(JNIEnv *jne, jclass jc, jint ji, jint ji1) {
  processing_instruction pi;
  
  glx_err err =
    glx_processing_instruction ((xs_string) ji, (xs_string) ji1, &pi);

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) pi;
}// Java_ProcessingInstruction_processingInstructionToItem()


/*
 * Node
 */

JNIEXPORT jint JNICALL Java_galapi_Node_nativeParent
(JNIEnv *jne, jclass jc, jint ji) {

  nodelist ndl;
  glx_err err = glx_parent ((node) ji, &ndl);

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) ndl;

}// Java_Node_nativeParent



JNIEXPORT jint JNICALL Java_galapi_Node_nativeChildren
(JNIEnv *jne, jclass jc, jint ji) {

  nodelist ndl;
  glx_err err = glx_children ((node) ji, &ndl);

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) ndl;

}// Java_Node_nativeChildren()



JNIEXPORT jint JNICALL Java_galapi_Node_nativeBaseURI
(JNIEnv *jne, jclass jc, jint ji) {
  atomicvalue_list al;
  glx_err err = glx_base_uri ((node) ji, &al); 

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);
  
  return (jint) al;
}// Java_Node_nativeBaseURI


JNIEXPORT jint JNICALL Java_galapi_Node_nativeNodeName
(JNIEnv *jne, jclass jc, jint ji) {
  atomicvalue_list al;
  glx_err err = glx_node_name ((node) ji, &al); 

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);
  
  return (jint) al;
}// Java_Node_nativeBaseURI


JNIEXPORT jstring JNICALL
Java_galapi_Node_nativeNodeKind (JNIEnv *jne, jclass jc, jint ji) {
  char *s;
  glx_err err = glx_node_kind ((item) ji, &s);
  if (0 != err)
    throw_exception (jne, "java/lang/RuntimeException", glx_error_string, err);
  
  return (*jne)->NewStringUTF(jne, s);
}// Java_Item_nativeString()


JNIEXPORT jint JNICALL Java_galapi_Node_nativeTypedValue
(JNIEnv *jne, jclass jc, jint ji) {

  atomicvalue_list a;
  glx_err err = glx_typed_value ((node) ji, &a);

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);
  
  return (jint) a;

}// Java_Node_nativeTypedValue


JNIEXPORT jint JNICALL Java_galapi_Node_nativeAttributes
(JNIEnv *jne, jclass jc, jint ji) {

  attribute_list a;
  glx_err err = glx_attributes ((node) ji, &a);

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) a;

}// Java_Node_nativeAttributes



/*
 * class ExternalContext
 */

JNIEXPORT jint JNICALL 
Java_galapi_ExternalContext_externalContextToItem
(JNIEnv * jne, jclass jc, jint ctxt, 
 jobjectArray jarr, jintArray values) {

  jint *p = (*jne)->GetIntArrayElements (jne, values, NULL);
  //jsize n = (*jne)->GetArrayLength (jne, values);

  jsize sz = (*jne)->GetArrayLength (jne, jarr);
  char **a = getStringArray (jne, jarr);
  external_context ec;
  itemlist il = (itemlist) ctxt;
  

  glx_err err =
    glx_build_external_context (il, a, (itemlist*) p, sz, &ec);

    
  releaseStringArray (jne, jarr, a);
  (*jne)->ReleaseIntArrayElements (jne, values, p, 0); 

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) ec;  
  
}// Java_galapi_ExternalContext_externalContextToItem()



/*
 * Galax static methods
 */

JNIEXPORT jint JNICALL Java_galapi_Galax_nativeLoadDocument
(JNIEnv *jne, jclass jc, jstring js) {

  nodelist ndl;      
  const char *s = (*jne)->GetStringUTFChars (jne, js, NULL);

  glx_err err = glx_load_document ((char *) s, &ndl);


  (*jne)->ReleaseStringUTFChars (jne, js, s);    
  
#ifdef DEBUG
  fprintf (stderr, "LoadDocument error code=%d\n", err);
  fflush (stderr);
#endif
  

  if (0 != err) {    
    throw_galapi_exception (jne, glx_error_string, err);
  }
  
  return (jint) ndl;
  
}// Java_Galax_nativeLoadDocument()


JNIEXPORT jint JNICALL Java_galapi_Galax_nativeLoadDocumentValidate
(JNIEnv *jne, jclass jc, jint mc, jstring js) {
  
    const char *s = (*jne)->GetStringUTFChars (jne, js, NULL); 
    nodelist ndl;
    glx_err err = glx_load_document_validate ((module_context)mc, (char*) s, &ndl);
    (*jne)->ReleaseStringUTFChars (jne, js, s);    

    if (0 != err)
      throw_galapi_exception (jne, glx_error_string, err);
    
    return (jint) ndl;
  
}// Java_Galax_nativeLoadDocumentValidate()


JNIEXPORT jstring JNICALL Java_galapi_Galax_nativeSerializeToString
(JNIEnv *jne, jclass jc, jint ji) {

  char *s;
  glx_err err = glx_serialize_to_string ((itemlist) ji, &s); 

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (*jne)->NewStringUTF(jne, s);  

}// Java_Galax_nativeSerializeToString()


JNIEXPORT void JNICALL Java_galapi_Galax_nativeSerializeToOutputChannel
(JNIEnv *jne, jclass jc, jint ji) {

  glx_err err = 
    glx_serialize_to_output_channel ((itemlist) ji);

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

}// Java_Galax_nativeSerializeToOutputChannel()


JNIEXPORT void JNICALL Java_galapi_Galax_nativeSerializeToFile
(JNIEnv *jne, jclass jc, jstring js, jint ji) {

    const char *s = (*jne)->GetStringUTFChars (jne, js, NULL); 
    glx_err err = glx_serialize_to_file ((char *) s, (itemlist)ji);
    (*jne)->ReleaseStringUTFChars (jne, js, s);

    if (0 != err)
      throw_galapi_exception (jne, glx_error_string, err);

}// Java_Galax_nativeSerializeToFile()


/*
 * Galax - evaluation & query
 *
 */


JNIEXPORT jint JNICALL Java_galapi_Galax_nativeDefaultProcessingContext
(JNIEnv * jne, jclass jc) {
  processing_context pc;
  glx_err err = glx_default_processing_context (&pc);
  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);
  return (jint) pc;
}

JNIEXPORT jint JNICALL Java_galapi_Galax_nativeLoadStandardLibrary
(JNIEnv *jne, jclass jc, jint ji) {
  module_context mc;
  glx_err err = glx_load_standard_library ((processing_context) ji, &mc);
  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) mc;
}

JNIEXPORT jint JNICALL Java_galapi_Galax_nativeImportLibraryModule
(JNIEnv *jne, jclass jc, jint ji, jstring js) {
  const char *s = (*jne)->GetStringUTFChars (jne, js, NULL); 
  module_context mc;

  glx_err err = glx_import_library_module ((module_context) ji, 
					   (char *)s, 
					   &mc);
  
  (*jne)->ReleaseStringUTFChars (jne, js, s);
  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) mc;
}

JNIEXPORT jint JNICALL Java_galapi_Galax_nativeImportMainModule
(JNIEnv *jne, jclass jc, jint ji, jstring js) {

  const char *s = (*jne)->GetStringUTFChars (jne, js, NULL); 
  query_context qc;
  glx_err err = glx_import_main_module ((module_context) ji, 
					(char *)s,
					&qc);
  (*jne)->ReleaseStringUTFChars (jne, js, s);
  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) qc;  
}

JNIEXPORT jint JNICALL Java_galapi_Galax_importMainModuleFromString
(JNIEnv *jne, jclass jc, jint ji, jstring js) {

  const char *s = (*jne)->GetStringUTFChars (jne, js, NULL); 
  query_context qc;
  glx_err err = 
    glx_import_main_module_from_string ((module_context) ji, 
					(char *)s,
					&qc);
  (*jne)->ReleaseStringUTFChars (jne, js, s);
  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) qc;

}



JNIEXPORT jint JNICALL Java_galapi_Galax_nativeAddExternalContext
(JNIEnv *jne, jclass jc, jint mc, jint ec) {

  module_context i;
  glx_err err = glx_add_external_context ((module_context) mc, 
					  (external_context) ec,
					  &i);
  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) i;  

}



JNIEXPORT jint JNICALL Java_galapi_Galax_nativeEvalGlobalVariables
(JNIEnv *jne, jclass jc, jint ji) {
  module_context mc;
  glx_err err = glx_eval_global_variables ((module_context) ji, &mc);
  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) mc;  
}


JNIEXPORT jint JNICALL Java_galapi_Galax_nativeEvalStatementFromString
(JNIEnv *jne, jclass jc, jint ji, jstring js) {

  const char *s = (*jne)->GetStringUTFChars (jne, js, NULL); 
  itemlist il;
  glx_err err = glx_eval_statement_from_string ((module_context) ji, (char *)s, &il);
  (*jne)->ReleaseStringUTFChars (jne, js, s);
  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);
  return (jint) il;  

}


JNIEXPORT jint JNICALL Java_galapi_Galax_nativeEvalQueryFunction
(JNIEnv *jne, jclass jc, jint c, jstring f, jintArray params) {

  const char *s = (*jne)->GetStringUTFChars (jne, f, NULL); 
  jint *p = (*jne)->GetIntArrayElements (jne, params, NULL);
  jsize n = (*jne)->GetArrayLength (jne, params);
  itemlist il;
  glx_err err = glx_eval_query_function ((module_context) c,
					 (char *)s,
					 (itemlist*)p,
					 (int)n,
					 &il);
  // I don't understand the following line, gcc raises a warning.
  // - Jerome
  //
  // I think I had forgotten a cast somewhere. Anyway, I tried
  // to rewrite properly all these functions.
  // - Nicola
  //

  (*jne)->ReleaseIntArrayElements (jne, params, p, 0); 
  (*jne)->ReleaseStringUTFChars (jne, f, s);

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) il;

}


JNIEXPORT jint JNICALL Java_galapi_Galax_nativeEvalStatements
(JNIEnv *jne, jclass hc, jint ji) {

  itemlist il;
  glx_err err = glx_eval_statements ((query_context) ji, &il);
  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) il;
}


JNIEXPORT jint JNICALL Java_galapi_Galax_nativeEvalStatementWithContextItemFromDocument
(JNIEnv *jne, jclass jc, jstring fname, jstring q) {

  const char *file_name = (*jne)->GetStringUTFChars (jne, fname, NULL); 
  const char *query_str = (*jne)->GetStringUTFChars (jne, q, NULL);   
  itemlist il;
  glx_err err = 
    glx_eval_statement_with_context_item_from_document ( (char*) file_name,
							 (char*) query_str,
							 &il);
  (*jne)->ReleaseStringUTFChars (jne, fname, file_name);
  (*jne)->ReleaseStringUTFChars (jne, q, query_str);
  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) il;  
  
}


JNIEXPORT jint JNICALL Java_galapi_Galax_nativeEvalStatementWithContextItemFromXML
(JNIEnv *jne, jclass jc, jint it, jstring q) {

  item it_ctxt = (item) it;
  const char *query_str = (*jne)->GetStringUTFChars (jne, q, NULL);   
  itemlist il;
  glx_err err = glx_eval_statement_with_context_item_from_xml ( it_ctxt,
								(char*) query_str,
								&il );
  (*jne)->ReleaseStringUTFChars (jne, q, query_str);
  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) il;
}

JNIEXPORT jint JNICALL Java_galapi_Galax_nativeEvalStatementFromFileWithContextItemFromXML
(JNIEnv *jne, jclass jc, jint it, jstring q) {

  item it_ctxt = (item) it;
  const char *file_name = (*jne)->GetStringUTFChars (jne, q, NULL);   
  itemlist il;
  glx_err err = glx_eval_statement_from_file_with_context_item_from_xml ( it_ctxt,
								(char*) file_name,
								&il );
  (*jne)->ReleaseStringUTFChars (jne, q, file_name);
  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) il;
}



JNIEXPORT jint JNICALL Java_galapi_Galax_nativeEvalStatementWithVariablesFromXML
(JNIEnv *jne, jclass jc, jobjectArray jarr, jintArray jvals, jstring q) {
  
  const char *query_str = (*jne)->GetStringUTFChars (jne, q, NULL);   
  itemlist il;
  

  char **vnames = getStringArray (jne, jarr);
  jint *v = (*jne)->GetIntArrayElements (jne, jvals, NULL);
  jsize sz = (*jne)->GetArrayLength (jne, jarr);

  glx_err err = glx_eval_statement_with_variables_from_xml (vnames, 
							    (itemlist*) v, 
							    (int) sz,
							    (char*) query_str,
							    &il);
  
  releaseStringArray (jne, jarr, vnames);
  (*jne)->ReleaseIntArrayElements (jne, jvals, v, 0); 
  (*jne)->ReleaseStringUTFChars (jne, q, query_str);

  if (0 != err)
    throw_galapi_exception (jne, glx_error_string, err);

  return (jint) il;
}


