(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                            XML Query Engine                         *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: norm_context.ml,v 1.2 2003/04/21 16:08:47 simeon Exp $ *)

open Xquery_ast
open Error

type norm_context = 
  { nsenv : Namespace.nsenv ;
    schema : Schema.schema ; 
    fun_sigs : ((rfname * int), (rfname * cfunction_signature)) Hashtbl.t } 

let build_norm_context nsenv schema siglist = 
  let norm_context =
    { nsenv = nsenv; 
      schema = schema; 
      fun_sigs = Hashtbl.create 167 }
  in
  let _ =
    List.iter 
      (fun ((fname, arity), (tfname, cfunc_signature)) ->
	let arity = List.length (fst cfunc_signature) in
(* print_string("add "^(Namespace.string_of_rqname fname)^" "^(string_of_int arity)^"\n");  *)
	Hashtbl.add norm_context.fun_sigs (fname,arity) (tfname,cfunc_signature)) siglist
  in
  norm_context 

(* We need to keep the hashtable of function signatures in tact. *)

let copy_norm_context_with_sigs ctxt nsenv schema = 
  let norm_context =
    { nsenv = nsenv;
      schema = schema; 
      fun_sigs = Hashtbl.copy ctxt.fun_sigs }
  in norm_context
  
let copy_norm_context ctxt = 
  let norm_context =
    { nsenv = ctxt.nsenv;
      schema = ctxt.schema; 
      fun_sigs = Hashtbl.copy ctxt.fun_sigs }
  in norm_context

let add_sigs_to_norm_context context siglist =
  let norm_context = copy_norm_context context in 
  let _ =
    List.iter 
      (fun ((fname, arity), (tfname, cfunc_signature)) ->
	let arity = List.length (fst cfunc_signature) in
(*      print_string ("Add "^(Namespace.string_of_rqname fname)^"\n");  *)
	Hashtbl.add norm_context.fun_sigs (fname,arity) (tfname, cfunc_signature)) siglist
  in
  norm_context

let nsenv_from_norm_context context = context.nsenv
let schema_from_norm_context context = context.schema

let one_sig_from_norm_context context (f, i) fi = 
  let l = Hashtbl.find_all context.fun_sigs (f,i) in
  match l with
  | [] -> raise (Query(Undefined(
		       " Signature of function " ^ (Namespace.string_of_rqname f) ^
		       " with arity " ^ (string_of_int i) ^
		       " undefined")))
  | [(f,s)] -> s
  | _ -> raise (Query(Internal_Error (" Function " ^ (Namespace.string_of_rqname f) ^
				      " with arity " ^ (string_of_int i) ^
				      " has more than one (" ^ (string_of_int(List.length l)) ^ ") signature")))

