(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                            XML Query Engine                         *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: parse_context.ml,v 1.4 2003/06/27 00:05:08 simeon Exp $ *)

open Error

(* This module implements the parsing context *)

type parse_context =
    { parse_general_entities : (string,string) Hashtbl.t;
      parse_parameter_entities : (string,string) Hashtbl.t }

(* Note:
      From the XML 1.0 spec:

   	<!ENTITY lt     "&#38;#60;">
   	<!ENTITY gt     "&#62;">
   	<!ENTITY amp    "&#38;#38;">
   	<!ENTITY apos   "&#39;">
   	<!ENTITY quot   "&#34;">

   - Jerome
 *)

let built_in_general_entities =
  [ ("lt", "&#38;#60;");
    ("gt", "&#62;");
    ("amp", "&#38;#38;");
    ("apos", "&#39;");
    ("quot", "&#38;#34;") ]
    
let build_parse_context () =
  let general_entity_table = Hashtbl.create 167 in
  let parameter_entity_table = Hashtbl.create 167 in
  List.iter (fun (x,y) -> Hashtbl.add general_entity_table x y) built_in_general_entities;
  { parse_general_entities = general_entity_table;
    parse_parameter_entities = parameter_entity_table }

let add_general_entity_to_parse_context parse_context x y =
  Hashtbl.add parse_context.parse_general_entities x y

let add_parameter_entity_to_parse_context parse_context x y =
  Hashtbl.add parse_context.parse_parameter_entities x y

let get_general_entity parse_context fi x =
  try
    Hashtbl.find parse_context.parse_general_entities x
  with
  | _ ->
      raise (Query (Parsing (fi,"General entity: '&"^x^";' not found")))

