(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                              XQuery Engine                          *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: path_analysis.ml,v 1.1 2003/05/21 15:03:49 simeon Exp $ *)

(* Module: Path_analysis
   Description:
     This module implements the path analysis algorithm used by
     projection.
*)

(* Note:
     This is out of date and does not work since Galax version
     0.3.0. This needs to be updated by Amelie.
   - Jerome *)

open Format

open Error
open Xquery_ast

open Path_struct
open Path_structutil
open Optim_vars
open Analysis_context
open Function_analysis


(* -------------------------------------------------------------------------
   Accesses the document id from the document function
-------------------------------------------------------------------------- *)

let get_docid el =
  match el with
  | [] -> None
  | [e] ->
      begin
	match e.pcexpr_desc with
	| Xquery_ast.CEScalar (Datatypes.AtomicString s) ->
	    Some s
	| _ -> None
      end
  | _ -> None


(* ------------------------------------------------------------------
   Serialize function: 
       Add subtrees to return
       Combine retrun and using
------------------------------------------------------------------- *)

let serialize (ps,ups) =
  let ps_s = imposes_subtree ps
  in Gmisc.remove_duplicates (ps_s @ ups)


(* ------------------------------------------------------------------
   Path Analysis function: 
      Without optimization
------------------------------------------------------------------- *)


let rec cexpr_path_analysis analysis_context cexpr =
  match cexpr.pcexpr_desc with
      (* I did not check this. This should do almost like optimization
         except that cexpr2 always returns a boolean - Jerome *)
  | CESome (dto, var, cexpr1, cexpr2)
  | CEEvery (dto, var, cexpr1, cexpr2) ->
      let (path_seq1, used_path_seq1) = cexpr_path_analysis analysis_context cexpr1 in
      let analysis_context' = add_var_path analysis_context var path_seq1 in
      let (path_seq2, used_path_seq2) = cexpr_path_analysis analysis_context' cexpr2 in
      (path_seq2,path_seq1 @ used_path_seq1 @ used_path_seq2)

	(* Note this is without optimization. Optimization tries to
   	   remove path_seq1 from the second part of the returned
   	   pair. - Jerome and Amelie *)

  | CEFor (dto, var, ovar', cexpr1, cexpr2) ->
      let (path_seq1, used_path_seq1) = cexpr_path_analysis analysis_context cexpr1 in
      let analysis_context' = add_var_path analysis_context var path_seq1 in
      let analysis_context'' =
	match ovar' with
	| None -> analysis_context'
	| Some var' ->
	    add_var_path analysis_context' var' []
      in
      let (path_seq2, used_path_seq2) = cexpr_path_analysis analysis_context'' cexpr2 in
      (path_seq2,path_seq1 @ used_path_seq1 @ used_path_seq2)

	(* Note this is without optimization. Optimization tries to
   	   remove path_seq1 from the second part of the returned
   	   pair. - Jerome and Amelie *)

  | CEForTuple (var, cexpr1, cexpr2) ->
      let (path_seq1, used_path_seq1) = cexpr_path_analysis analysis_context cexpr1 in
      let analysis_context' = add_var_path analysis_context var path_seq1 in
      let (path_seq2, used_path_seq2) = cexpr_path_analysis analysis_context' cexpr2 in
      (path_seq2,path_seq1 @ used_path_seq1 @ used_path_seq2)

	(* Note this is without optimization. Optimization tries to
   	   remove path_seq1 from the second part of the returned
   	   pair. - Jerome and Amelie *)

  | CELet (dto, var, cexpr1, cexpr2) ->
      let (path_seq1, used_path_seq1) = cexpr_path_analysis analysis_context cexpr1 in
      let analysis_context' = add_var_path analysis_context var path_seq1 in
      let (path_seq2, used_path_seq2) = cexpr_path_analysis analysis_context' cexpr2 in
      (path_seq2, used_path_seq1 @ used_path_seq2)

	(* Note this is without optimization. Optimization tries to
   	   remove path_seq1 from the second part of the returned
   	   pair. - Jerome and Amelie *)

  | CEVar var ->
      let path_sequence = get_var_path analysis_context var in
      (path_sequence, [])

  | CESeq (cexpr1, cexpr2) ->
      let (path_seq1, used_path_seq1) = cexpr_path_analysis analysis_context cexpr1
      and (path_seq2, used_path_seq2) = cexpr_path_analysis analysis_context cexpr2 in
      (path_seq1 @ path_seq2, used_path_seq1 @ used_path_seq2)

  | CEIf (cexpr1, cexpr2, cexpr3) ->
      let (path_seq1, used_path_seq1) = cexpr_path_analysis analysis_context cexpr1
      and (path_seq2, used_path_seq2) = cexpr_path_analysis analysis_context cexpr2
      and (path_seq3, used_path_seq3) = cexpr_path_analysis analysis_context cexpr3 in
      (path_seq2 @ path_seq3, path_seq1 @ used_path_seq1 @ used_path_seq2 @ used_path_seq3)

  | CECast (cexpr1, model) ->
      let (path_seq1, used_path_seq1) = cexpr_path_analysis analysis_context cexpr1 in
      ([], path_seq1 @ used_path_seq1)
 
  | CECastable (cexpr1, model) ->
      let (path_seq1, used_path_seq1) = cexpr_path_analysis analysis_context cexpr1 in
      ([], path_seq1 @ used_path_seq1)
 
  | CEValidate (cexpr1, model) ->
      let (path_seq1, used_path_seq1) = cexpr_path_analysis analysis_context cexpr1 in
      ([], path_seq1 @ used_path_seq1)

  | CEForwardAxis (axis, cnode_test) ->
      let path_seq = get_var_path analysis_context Xquery_astutil.dot in
      let axis_fun (docid,(path,subtree)) = (docid, (path @ [(axis, cnode_test)], subtree)) in
      (List.map axis_fun path_seq, [])

  | CEReverseAxis (axis, cnode_test) ->
      let path_seq = get_var_path analysis_context Xquery_astutil.dot in
      let axis_fun (docid,(path,subtree)) = (docid, (path @ [(axis, cnode_test)], subtree)) in
      (List.map axis_fun path_seq, [])

  | CEUnordered cexpr1 ->
      cexpr_path_analysis analysis_context cexpr1 

  | CEText _ -> ([],[])
	
  | CEScalar _ -> ([],[])

  | CEEmpty -> ([],[])

  | CEComment _ -> ([],[])

  | CEPI _ -> ([],[])

  | CEDocument cexpr1 ->
      let (path_seq1, used_path_seq1) = cexpr_path_analysis analysis_context cexpr1 in
      let path_seq1' = imposes_subtree path_seq1 in
      ([], path_seq1' @ used_path_seq1)

  | CEElem (symbol, cexpr1) ->
      let (path_seq1, used_path_seq1) = cexpr_path_analysis analysis_context cexpr1 in
      let path_seq1' = imposes_subtree path_seq1 in
      ([], path_seq1' @ used_path_seq1)

  | CEAttr (symbol, cexpr1) ->
      let (path_seq1, used_path_seq1) = cexpr_path_analysis analysis_context cexpr1 in
      let path_seq1' = imposes_subtree path_seq1 in  (*do we need the subtree here? should not pose problem. Amelie*)
      ([], path_seq1' @ used_path_seq1)

  | CETextComputed cexpr1 ->
      let (path_seq1, used_path_seq1) = cexpr_path_analysis analysis_context cexpr1 in
      let path_seq1' = imposes_subtree path_seq1 in  (*do we need the subtree here? should not pose problem. Amelie*)
      ([], path_seq1' @ used_path_seq1)

  | CEAnyElem (cexpr1, cexpr2) ->
      let (path_seq1, used_path_seq1) = cexpr_path_analysis analysis_context cexpr1 
      and (path_seq2, used_path_seq2) = cexpr_path_analysis analysis_context cexpr2 in  
      let path_seq1' = imposes_subtree path_seq1  (*do we need the subtree here? should not pose problem. Amelie*)
      and path_seq2' = imposes_subtree path_seq2 in
      ([], path_seq1' @ path_seq2' @ used_path_seq1 @ used_path_seq2)
      
  | CEAnyAttr (cexpr1, cexpr2) ->
      let (path_seq1, used_path_seq1) = cexpr_path_analysis analysis_context cexpr1 
      and (path_seq2, used_path_seq2) = cexpr_path_analysis analysis_context cexpr2 in  
      let path_seq1' = imposes_subtree path_seq1  (*do we need the subtree here? should not pose problem. Amelie*)
      and path_seq2' = imposes_subtree path_seq2 in  (*do we need the subtree here? should not pose problem. Amelie*)
      ([], path_seq1' @ path_seq2' @ used_path_seq1 @ used_path_seq2)

  | CETypeswitch (cexpr1, tsclause) ->
      let (path_seq1, used_path_seq1) = cexpr_path_analysis analysis_context cexpr1 in
      let (path_seq2, used_path_seq2) = typeswitchclauses_path_analysis analysis_context tsclause path_seq1 in
      (path_seq2, path_seq1 @ used_path_seq1 @ used_path_seq2)
	
  | CESort (v, cexpr1, stablekind, cexpr2, sortkind, emptysortkind) ->
      let (path_seq1, used_path_seq1) = cexpr_path_analysis analysis_context cexpr1 in
      let analysis_context' = add_var_path analysis_context v path_seq1 in
      let (path_seq2, used_path_seq2) = cexpr_path_analysis analysis_context' cexpr2 in
      (path_seq1, path_seq2 @ used_path_seq1 @ used_path_seq2)

  | CEError cexpr ->
      let (path_seq1, used_path_seq1) = cexpr_path_analysis analysis_context cexpr in
      ([], path_seq1 @ used_path_seq1)

  | CETreat (cexpr, model) ->
      cexpr_path_analysis analysis_context cexpr

  | CESubtype _ -> ([],[])
  
  | CEIntersects _ -> ([],[])

  | CEKeyRef _ -> ([],[])

  | CECreateTuple record_list ->
      let paths_list = List.map (fun (r,cexpr) -> cexpr_path_analysis analysis_context cexpr) record_list in
      let (return_paths,used_paths) = List.split paths_list in
      let return_paths' = List.concat return_paths in
      let used_paths' = List.concat used_paths in
      (return_paths', used_paths')

  | CEAccessTuple (cexpr,rname) ->
      cexpr_path_analysis analysis_context cexpr
 
  | CEApp (rfname, el) ->
      begin
	if (rfname = (Namespace.fn_uri,"document"))
	then
	  let odocid = get_docid el in
	  match odocid with
	  | Some docid ->
	      ([(docid,([],NoSubtree))],[])
	  | None ->
	      let paths_list = List.map (cexpr_path_analysis analysis_context) el in
	      let (return_paths,used_paths) = List.split paths_list in
	      let return_paths' = List.concat return_paths in
	      let used_paths' = List.concat used_paths in
	      let return_paths_subtree = imposes_subtree return_paths' in
	      (return_paths_subtree, used_paths')
	else
	 let paths_list = List.map (cexpr_path_analysis analysis_context) el in
         let (return_paths,used_paths) = List.split paths_list in
         let return_paths' = List.concat return_paths in
         let used_paths' = List.concat used_paths in
	 match get_fun_analysis_type rfname with 
	 | UsedOnly ->
	     ([],used_paths')
	 | UsedReturnSimple ->
	      ([],return_paths' @ used_paths')
	 | UsedReturnSubtree ->
	     let return_paths_subtree = imposes_subtree return_paths' in
	     ([], return_paths_subtree @ used_paths')
	 | ReturnsPaths ->
	     (return_paths', used_paths')
	 | ReturnsDefault ->
	     let return_paths_subtree = imposes_subtree return_paths' in
	     (return_paths_subtree, used_paths')
      end

and typeswitchclauses_path_analysis analysis_context tsclause input_path_seq = 
  match tsclause with
  | [] ->
      ([],[])
  | (_, ovar, cexpr1) :: tsclause_rest ->
      begin
	match ovar with
	|  Some var -> 
	    let analysis_context' = add_var_path analysis_context var input_path_seq in
	    let (path_seq1, used_path_seq1) = cexpr_path_analysis analysis_context' cexpr1 in
	    let (path_seq2, used_path_seq2) = typeswitchclauses_path_analysis analysis_context tsclause_rest input_path_seq in
	    (path_seq1 @ path_seq2, used_path_seq1 @ used_path_seq2)
	| None ->
	    let (path_seq1, used_path_seq1) = cexpr_path_analysis analysis_context cexpr1 in
	    let (path_seq2, used_path_seq2) = typeswitchclauses_path_analysis analysis_context tsclause_rest input_path_seq in
	    (path_seq1 @ path_seq2, used_path_seq1 @ used_path_seq2)
      end


(* ---------------------------------------------------------------------------
   Path Analysis with optimization
---------------------------------------------------------------------------- *)


let rec cexpr_path_analysis_optim analysis_context cexpr =
  match cexpr.pcexpr_desc with
  | CESome (dto, var, cexpr1, cexpr2)
  | CEEvery (dto, var, cexpr1, cexpr2) ->
      let (path_seq1, used_path_seq1, optim_var1 ) = cexpr_path_analysis_optim analysis_context cexpr1 in
      let analysis_context' = add_var_path analysis_context var path_seq1 in
      let (path_seq2, used_path_seq2, optim_var2) = cexpr_path_analysis_optim analysis_context' cexpr2 in
      let optim_var3 = remove var optim_var2 in
      (path_seq2,path_seq1 @ used_path_seq1 @ used_path_seq2, (union optim_var1 optim_var3))

  | CEFor (dto, var, ovar', cexpr1, cexpr2) ->
      let (path_seq1, used_path_seq1, optim_var1 ) = cexpr_path_analysis_optim analysis_context cexpr1 in
      let analysis_context' = add_var_path analysis_context var path_seq1 in
      let analysis_context'' =
	match ovar' with
	| None -> analysis_context'
	| Some var' ->
	    add_var_path analysis_context' var' []
      in
      let (path_seq2, used_path_seq2, optim_var2) = cexpr_path_analysis_optim analysis_context'' cexpr2 in
      if(member var optim_var2)
      then 
	let optim_var3 = remove var optim_var2 in
	(path_seq2,used_path_seq1 @ used_path_seq2, (union optim_var1 optim_var3))
      else
	(path_seq2,path_seq1 @ used_path_seq1 @ used_path_seq2, (union optim_var1 optim_var2))

  | CEForTuple (var, cexpr1, cexpr2) ->
      let (path_seq1, used_path_seq1, optim_var1 ) = cexpr_path_analysis_optim analysis_context cexpr1 in
      let analysis_context' = add_var_path analysis_context var path_seq1 in
      let (path_seq2, used_path_seq2, optim_var2) = cexpr_path_analysis_optim analysis_context' cexpr2 in
      if(member var optim_var2)
      then 
	let optim_var3 = remove var optim_var2 in
	(path_seq2,used_path_seq1 @ used_path_seq2, (union optim_var1 optim_var3))
      else
	(path_seq2,path_seq1 @ used_path_seq1 @ used_path_seq2, (union optim_var1 optim_var2))

  | CELet (dto, var, cexpr1, cexpr2) ->
      let (path_seq1, used_path_seq1, optim_var1) = cexpr_path_analysis_optim analysis_context cexpr1 in
      let analysis_context' = add_var_path analysis_context var path_seq1 in
      let (path_seq2, used_path_seq2, optim_var2) = cexpr_path_analysis_optim analysis_context' cexpr2 in
      let optim_var3 = remove var optim_var2 in
      (path_seq2, used_path_seq1 @ used_path_seq2, (optim_var3))

  | CEVar var ->
      let path_sequence = get_var_path analysis_context var in
      (path_sequence, [], single_optimvars var)

  | CESeq (cexpr1, cexpr2) ->
      let (path_seq1, used_path_seq1, optim_var1) = cexpr_path_analysis_optim analysis_context cexpr1
      and (path_seq2, used_path_seq2, optim_var2) = cexpr_path_analysis_optim analysis_context cexpr2 in
      (path_seq1 @ path_seq2, used_path_seq1 @ used_path_seq2, (intersection optim_var1 optim_var2))

  | CEIf (cexpr1, cexpr2, cexpr3) ->
      let (path_seq1, used_path_seq1, optim_var1) = cexpr_path_analysis_optim analysis_context cexpr1
      and (path_seq2, used_path_seq2, optim_var2) = cexpr_path_analysis_optim analysis_context cexpr2
      and (path_seq3, used_path_seq3, optim_var3) = cexpr_path_analysis_optim analysis_context cexpr3 in
      (path_seq2 @ path_seq3, path_seq1 @ used_path_seq1 @ used_path_seq2 @ used_path_seq3, intersection optim_var2 optim_var3)

  | CECast (cexpr1, model) ->
      let (path_seq1, used_path_seq1, optim_var1) = cexpr_path_analysis_optim analysis_context cexpr1 in
      ([], path_seq1 @ used_path_seq1, optim_var1)
 
  | CECastable (cexpr1, model) ->
      let (path_seq1, used_path_seq1, optim_var1) = cexpr_path_analysis_optim analysis_context cexpr1 in
      ([], path_seq1 @ used_path_seq1, optim_var1)
 
  | CEValidate (cexpr1, model) ->
      let (path_seq1, used_path_seq1, optim_var1) = cexpr_path_analysis_optim analysis_context cexpr1 in
      ([], path_seq1 @ used_path_seq1, optim_var1)

  | CEForwardAxis (axis, cnode_test) ->
      let path_seq = get_var_path analysis_context Xquery_astutil.dot in
      let axis_fun (docid,(path,subtree)) = (docid, (path @ [(axis, cnode_test)], subtree)) in
      (List.map axis_fun path_seq, [], single_optimvars Xquery_astutil.dot)

  | CEReverseAxis (axis, cnode_test) ->
      let path_seq = get_var_path analysis_context Xquery_astutil.dot in
      let axis_fun (docid,(path,subtree)) = (docid, (path @ [(axis, cnode_test)], subtree)) in
      (List.map axis_fun path_seq, [], single_optimvars Xquery_astutil.dot)

  | CEUnordered cexpr1 ->
      cexpr_path_analysis_optim analysis_context cexpr1 

  | CEText _ -> ([],[],empty_optimvars)
	
  | CEScalar _ -> ([],[],empty_optimvars)

  | CEEmpty -> ([],[], all_optimvars)

  | CEComment _ -> ([],[],empty_optimvars)

  | CEPI _ -> ([],[],empty_optimvars)

  | CEDocument cexpr1 ->
      let (path_seq1, used_path_seq1, optim_var1) = cexpr_path_analysis_optim analysis_context cexpr1 in
      let path_seq1' = imposes_subtree path_seq1 in
      ([], path_seq1' @ used_path_seq1, empty_optimvars)

  | CEElem (symbol, cexpr1) ->
      let (path_seq1, used_path_seq1, optim_var1) = cexpr_path_analysis_optim analysis_context cexpr1 in
      let path_seq1' = imposes_subtree path_seq1 in
      ([], path_seq1' @ used_path_seq1, empty_optimvars)

  | CEAttr (symbol, cexpr1) ->
      let (path_seq1, used_path_seq1, optim_var1) = cexpr_path_analysis_optim analysis_context cexpr1 in
      let path_seq1' = imposes_subtree path_seq1 in  (*do we need the subtree here? should not pose problem. Amelie*)
      ([], path_seq1' @ used_path_seq1, empty_optimvars)

  | CETextComputed cexpr1 ->
      let (path_seq1, used_path_seq1, optim_var1) = cexpr_path_analysis_optim analysis_context cexpr1 in
      let path_seq1' = imposes_subtree path_seq1 in  (*do we need the subtree here? should not pose problem. Amelie*)
      ([], path_seq1' @ used_path_seq1, empty_optimvars)

  | CEAnyElem (cexpr1, cexpr2) ->
      let (path_seq1, used_path_seq1, optim_var1) = cexpr_path_analysis_optim analysis_context cexpr1 
      and (path_seq2, used_path_seq2, optim_var2) = cexpr_path_analysis_optim analysis_context cexpr2 in  
      let path_seq1' = imposes_subtree path_seq1  (*do we need the subtree here? should not pose problem. Amelie*)
      and path_seq2' = imposes_subtree path_seq2 in
      ([], path_seq1' @ path_seq2' @ used_path_seq1 @ used_path_seq2, empty_optimvars)
  
  | CEAnyAttr (cexpr1, cexpr2) ->
      let (path_seq1, used_path_seq1, optim_var1) = cexpr_path_analysis_optim analysis_context cexpr1 
      and (path_seq2, used_path_seq2, optim_var2) = cexpr_path_analysis_optim analysis_context cexpr2 in  
      let path_seq1' = imposes_subtree path_seq1  (*do we need the subtree here? should not pose problem. Amelie*)
      and path_seq2' = imposes_subtree path_seq2 in  (*do we need the subtree here? should not pose problem. Amelie*)
      ([], path_seq1' @ path_seq2' @ used_path_seq1 @ used_path_seq2, empty_optimvars) 

  | CETypeswitch (cexpr1, tsclause) ->
      let (path_seq1, used_path_seq1, optim_var1) = cexpr_path_analysis_optim analysis_context cexpr1 in
      let (path_seq2, used_path_seq2, optim_var2) = ts_path_analysis_optim analysis_context tsclause path_seq1 in
      (path_seq2, path_seq1 @ used_path_seq1 @ used_path_seq2, optim_var2) 
	
  | CESort (v, cexpr1, stablekind, cexpr2, sortkind, emptysortkind) ->
      let (path_seq1, used_path_seq1, optim_var1) = cexpr_path_analysis_optim analysis_context cexpr1 in
      let analysis_context' = add_var_path analysis_context v path_seq1 in
      let (path_seq2, used_path_seq2, optim_var2) = cexpr_path_analysis_optim analysis_context' cexpr2 in
      (path_seq1, path_seq2 @ used_path_seq1 @ used_path_seq2, optim_var1)

  | CEError cexpr ->
      let (path_seq1, used_path_seq1, optim_var1) = cexpr_path_analysis_optim analysis_context cexpr in
      ([], path_seq1 @ used_path_seq1,optim_var1)

  | CETreat (cexpr, model) ->
      cexpr_path_analysis_optim analysis_context cexpr

  | CESubtype _ -> ([],[],empty_optimvars)
  
  | CEIntersects _ -> ([],[],empty_optimvars)

  | CEKeyRef _ -> ([],[],empty_optimvars)

  | CECreateTuple record_list ->
      let paths_list = List.map (fun (r,cexpr) -> cexpr_path_analysis_optim analysis_context cexpr) record_list in
      let (return_paths,used_paths,optim_vars) = Gmisc.triple_split paths_list in
      let return_paths' = List.concat return_paths in
      let used_paths' = List.concat used_paths in
      (return_paths', used_paths',empty_optimvars)

  | CEAccessTuple (cexpr,rname) ->
      cexpr_path_analysis_optim analysis_context cexpr
 
  | CEApp (rfname, el) ->
      begin
	if (rfname = (Namespace.fn_uri,"document"))
	then
	  let odocid = get_docid el in
	  match odocid with
	  | Some docid ->
	      ([(docid,([],NoSubtree))],[],empty_optimvars)
	  | None ->
	      let paths_list = List.map (cexpr_path_analysis_optim analysis_context) el in
	      let (return_paths,used_paths,optim_vars) = Gmisc.triple_split paths_list in
	      let return_paths' = List.concat return_paths in
	      let used_paths' = List.concat used_paths in
	      (* let optim_vars' = List.concat optim_vars in // use union here*)
	      let return_paths_subtree = imposes_subtree return_paths' in
	      (return_paths_subtree, used_paths',empty_optimvars)
	else
	  let paths_list = List.map (cexpr_path_analysis_optim analysis_context) el in
	  let (return_paths,used_paths,optim_vars) = Gmisc.triple_split paths_list in
	  let return_paths' = List.concat return_paths in
	  let used_paths' = List.concat used_paths in
	  (* let optim_vars' = List.concat optim_vars in // use union here*)
	  match get_fun_analysis_type rfname with 
	 | UsedOnly ->
	     ([],used_paths',empty_optimvars)
	 | UsedReturnSimple ->
	      ([],return_paths' @ used_paths',empty_optimvars)
	 | UsedReturnSubtree ->
	     let return_paths_subtree = imposes_subtree return_paths' in
	     ([], return_paths_subtree @ used_paths',empty_optimvars)
	 | ReturnsPaths ->
	     (return_paths', used_paths',empty_optimvars)
	 | ReturnsDefault ->
	     let return_paths_subtree = imposes_subtree return_paths' in
	     (return_paths_subtree, used_paths',empty_optimvars)
     
      end

and ts_path_analysis_optim analysis_context tsclause input_path_seq = 
  match tsclause with
  | [] ->
      ([],[],empty_optimvars)
  | (_, ovar, cexpr1) :: tsclause_rest ->
      begin
	match ovar with
	|  Some var -> 
	    let analysis_context' = add_var_path analysis_context var input_path_seq in
	    let (path_seq1, used_path_seq1, optim_var1) = cexpr_path_analysis_optim analysis_context' cexpr1 in
	    let (path_seq2, used_path_seq2, optim_var2) = ts_path_analysis_optim analysis_context tsclause_rest input_path_seq in
	    (path_seq1 @ path_seq2, used_path_seq1 @ used_path_seq2, (intersection optim_var1 optim_var2))
	| None ->
	    let (path_seq1, used_path_seq1, optim_var1) = cexpr_path_analysis_optim analysis_context cexpr1 in
	    let (path_seq2, used_path_seq2, optim_var2) = ts_path_analysis_optim analysis_context tsclause_rest input_path_seq in
	    (path_seq1 @ path_seq2, used_path_seq1 @ used_path_seq2, (intersection optim_var1 optim_var2))
      end 

(* ---------------------------------------------------------------------------
   Analysis for query prolog
---------------------------------------------------------------------------- *)

let query_functions_analysis analysis_context cfunction_def =
  (* Note: We will eventually have to do some analysis of the
       user-defined functions. - Jerome and Amelie *)
  (analysis_context,[])

let query_var_analysis analysis_context vardecl =
  match vardecl.pcvar_decl_desc with
  | (vname, cexpr) ->
      let (path_seq, used_path_seq) = cexpr_path_analysis analysis_context cexpr in
      let analysis_context' = add_var_path analysis_context vname path_seq in
      (analysis_context', used_path_seq)

let rec query_vars_analysis analysis_context vardecls =
  match vardecls with
  | [] -> (analysis_context,[])
  | vardecl :: vardecls' ->
      let (analysis_context',used_path_seq') = query_vars_analysis analysis_context vardecls' in
      let (analysis_context'',used_path_seq'') = query_var_analysis analysis_context' vardecl in
      (analysis_context'', used_path_seq' @ used_path_seq'')

let query_prolog_analysis analysis_context cqmodule =
  let (analysis_context',used_path_seq') = query_functions_analysis analysis_context cqmodule.pcmodule_functions in
  let (analysis_context'',used_path_seq'') = query_vars_analysis analysis_context' cqmodule.pcmodule_vars in
  (analysis_context'', used_path_seq' @ used_path_seq'')

let query_functions_analysis_optim analysis_context cfunction_def =
  (* Note: We will eventually have to do some analysis of the
       user-defined functions. - Jerome and Amelie *)
  (analysis_context,[])

let query_var_analysis_optim analysis_context vardecl =
  match vardecl.pcvar_decl_desc with
  | (vname, cexpr) ->
  let (path_seq, used_path_seq,_) = cexpr_path_analysis_optim analysis_context cexpr in
  let analysis_context' = add_var_path analysis_context vname path_seq in
  (analysis_context', used_path_seq)

let rec query_vars_analysis_optim analysis_context vardecls =
  match vardecls with
  | [] -> (analysis_context,[])
  | vardecl :: vardecls' ->
      let (analysis_context',used_path_seq') = query_vars_analysis_optim analysis_context vardecls' in
      let (analysis_context'',used_path_seq'') = query_var_analysis_optim analysis_context' vardecl in
      (analysis_context'', used_path_seq' @ used_path_seq'')

let query_prolog_analysis_optim analysis_context cqmodule =
  let (analysis_context',used_path_seq') = query_functions_analysis analysis_context cqmodule.pcmodule_functions in
  let (analysis_context'',used_path_seq'') = query_vars_analysis analysis_context' cqmodule.pcmodule_vars in
  (analysis_context'', used_path_seq' @ used_path_seq'')


(* ---------------------------------------------------------------------------
   Analysis for single statements
---------------------------------------------------------------------------- *)

let cstatement_path_analysis analysis_context cstatement =
  match cstatement.pcstatement_desc with
  | CSExpression ce ->
      cexpr_path_analysis analysis_context ce
  | CSUpdate cu ->
      raise (Query (Prototype "Analysis: Updates not supported yet"))

let cstatement_path_analysis_optim analysis_context cstatement =
  match cstatement.pcstatement_desc with
  | CSExpression ce ->
      cexpr_path_analysis_optim analysis_context ce
  | CSUpdate cu ->
      raise (Query (Prototype "Analysis: Updates not supported yet"))

(* ---------------------------------------------------------------------------
   Analysis for complete queries
---------------------------------------------------------------------------- *)

let query_path_analysis cqmodule =
  (* Dealing with the prolog *)
  let (analysis_context,used_path_seq) =
    query_prolog_analysis (build_analysis_context ()) cqmodule
  in

  (* Analysis for each expression in the module *)
  let statements = cqmodule.pcmodule_exprs in

  (* Merging the paths *)
  let paths_list = List.map (cstatement_path_analysis analysis_context) statements in
  let (return_paths,used_paths) = List.split paths_list in
  let return_paths' = List.concat return_paths in
  let used_paths' = List.concat used_paths in

  (* Final serialization *)
  serialize (return_paths', used_path_seq @ used_paths')


let query_path_analysis_optim cqmodule =
  (* Dealing with the prolog *)
  let (analysis_context,used_path_seq) =
    query_prolog_analysis_optim (build_analysis_context ()) cqmodule
  in

  (* Analysis for each expression in the module *)
  let statements = cqmodule.pcmodule_exprs in

  (* Merging the paths *)
  let paths_list = List.map (cstatement_path_analysis_optim analysis_context) statements in

  let (return_paths,used_paths,_) = Gmisc.triple_split paths_list in
  let return_paths' = List.concat return_paths in
  let used_paths' = List.concat used_paths in

  (* Final serialization *)
  serialize (return_paths', used_path_seq @ used_paths')


(* ---------------------------------------------------------------------------
   Printing of analysis results
---------------------------------------------------------------------------- *)


let print_intermediate_analysis ff (ps1,ps2) =
  begin
    fprintf ff "-------------------------------------\nRETURN\n";
    print_path_sequence ff ps1;
    fprintf ff "USING\n";
    print_path_sequence ff ps2;
    fprintf ff "-------------------------------------\n";
  end

let print_full_analysis ff ps =
  begin
    fprintf ff "-------------------------------------\nPATHS\n";
    print_path_sequence ff ps;
    fprintf ff "-------------------------------------\n";
  end

