(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                            XML Query Engine                         *)
(*                                                                     *)
(*  Copyright 2000, Lucent Technologies.                               *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: intersection.ml,v 1.9 2001/11/21 13:56:44 simeon Exp $ *)

open Sym
open Simple
open Xquery_ast
open Xquery_astutil
open Schemautil
open Error

(* Raw intersection *)

let intersect_unit m1 m2 =
  match (m1.pmodel_desc, m2.pmodel_desc) with
  | (TElemLocal (RElemSym l1,_,_), TElemLocal (RElemSym l2,_,_))
    ->
      if (relem_subtag l1 l2)
      then m1
      else
	if (relem_subtag l2 l1)
	then m2
	else raise Not_found
  | (TElemLocal (RElemSym l1,_,_), TElemRef (RElemSym l2))
    ->
      if (relem_subtag l1 l2)
      then m1
      else
	if (relem_subtag l2 l1)
	then m2
	else raise Not_found
  | (TElemRef (RElemSym l1), TElemLocal (RElemSym l2,_,_))
    ->
      if (relem_subtag l1 l2)
      then m1
      else
	if (relem_subtag l2 l1)
	then m2
	else raise Not_found
  | (TElemRef (RElemSym l1), TElemRef (RElemSym l2))
    ->
      if (relem_subtag l1 l2)
      then m1
      else
	if (relem_subtag l2 l1)
	then m2
	else raise Not_found
  | (TAttrLocal (RAttrSym l1,_), TAttrLocal (RAttrSym l2,_))
    ->
      if (rattr_subtag l1 l2)
      then m1
      else
	if (rattr_subtag l2 l1)
	then m2
	else raise Not_found
  | (TAttrLocal (RAttrSym l1,_), TAttrRef (RAttrSym l2))
    ->
      if (rattr_subtag l1 l2)
      then m1
      else
	if (rattr_subtag l2 l1)
	then m2
	else raise Not_found
  | (TAttrRef (RAttrSym l1), TAttrLocal (RAttrSym l2,_))
    ->
      if (rattr_subtag l1 l2)
      then m1
      else
	if (rattr_subtag l2 l1)
	then m2
	else raise Not_found
  | (TAttrRef (RAttrSym l1), TAttrRef (RAttrSym l2))
    ->
      if (rattr_subtag l1 l2)
      then m1
      else
	if (rattr_subtag l2 l1)
	then m2
	else raise Not_found
  | (TBase (STString), TBase (STString)) ->
      fmkmodel (TBase(STString)) Finfo.bogus
  | (TBase (STDecimal), TBase (STDecimal)) ->
      fmkmodel (TBase(STDecimal)) Finfo.bogus
  | (TBase (STFloat), TBase (STFloat)) ->
      fmkmodel (TBase(STFloat)) Finfo.bogus
  | (TBase (STBoolean), TBase (STBoolean)) ->
      fmkmodel (TBase(STBoolean)) Finfo.bogus
  | _ ->
   (* Format.printf "%s" ("Warning: intersection failed between type:\n"
			  ^ (Print.sprint_type "" t1)
			  ^ "\nand type:\n"
			  ^ (Print.sprint_type "" t2)); *)

   (* Intersection is only defined on unit types; TAttrRef, TElemRef, TAmp, etc.
      are not supported *)

      raise Not_found

let get_factor s m =
  let (ml,_,_) = sub_factor s m [] in ml

let rec sub_process_inter m1 ml2 =
  match ml2 with
  | [] -> []
  | m2 :: ml2' ->
      let more = sub_process_inter m1 ml2' in
      try
	(intersect_unit m1 m2) :: more
      with
      | Not_found ->
	  more

let rec process_inter ml1 ml2 =
  match ml1 with
  | [] -> []
  | m1 :: ml1' ->
      let more = process_inter ml1' ml2 in
      (sub_process_inter m1 ml2) @ more

let intersect_type m1 m2 s =
  let m1' = get_factor s m1
  and m2' = get_factor s m2 in
  union_types (process_inter m1' m2')
