(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                            XML Query Engine                         *)
(*                                                                     *)
(*  Copyright 2000, Lucent Technologies.                               *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: theta.ml,v 1.11 2002/09/22 21:25:10 simeon Exp $ *)

open Error
open Xquery_ast

(******************************************************)
(* This module implements operations on type mappings *)
(******************************************************)

(************************)
(* Subsumption mappings *)
(************************)

module OrderedTypeName =
  struct
    type t = Sym.rgroup_symbol
    let compare = compare
  end

module ThetaMapping = Map.Make(OrderedTypeName)

type theta = Sym.rgroup_symbol ThetaMapping.t

(* Operations on subsumption mappings *)

let empty = ThetaMapping.empty
    (* the empty mapping *)

let add x y th =
  try
    let y' = ThetaMapping.find x th in
    if (OrderedTypeName.compare y y') != 0
    then raise (Query (Mapping_Failure ("Subsumption failed: type "
					^ (Namespace.string_of_rqname(Sym.rgroup_name x))
					^ " mapped twice to "
					^ (Namespace.string_of_rqname(Sym.rgroup_name y))
					^ " and "
					^ (Namespace.string_of_rqname(Sym.rgroup_name y')))))
    else
      th
  with
  | Not_found ->
      ThetaMapping.add x y th
    (* adds a new mapping *)

let find x th =
  try
    ThetaMapping.find x th
  with
  | Not_found ->
      raise (Query (Mapping_Failure ("Type "
				     ^ (Namespace.string_of_rqname(Sym.rgroup_name x))
				     ^ " not currently mapped.")))
    (* finds a mapping if it exists or fails *)

let find_if x th =
  try
    ThetaMapping.find x th
  with
  | Not_found ->
      x
    (* finds a mapping or returns the original name *)

let reverse k th =
  let stack = ref [] in
  let keep x y =
    if (OrderedTypeName.compare x k) = 0
    then
      stack := y :: !stack
    else
      ()
  in
  ThetaMapping.iter keep th;
  !stack
    (* finds all the type names pointing to a given one *)

let merge th1 th2 =
  ThetaMapping.fold add th1 th2
    (* merges two mappings *)

let content th =
  let cons x y xs = (x,y) :: xs in
  ThetaMapping.fold cons th []

let print th =
  let content = content th in
  let print_binding (g1,g2) =
    Format.printf "%s << %s\n" (Sym.rgroup_prefix_string g1) (Sym.rgroup_prefix_string g2)
  in
  begin
    Format.printf "Subsumption mapping:\n";
    Format.printf "--------------------\n";
    List.iter print_binding content;
    Format.printf "--------------------\n";
  end

