(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                            XML Query Engine                         *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: fn_error.ml,v 1.1 2003/05/01 20:02:07 simeon Exp $ *)

(* Module Fn_error
   Description:
     This module implements support for the user-level fn:error()
     function.
*)

open Error

open Namespace
open Datamodel

type xquery_error_msg =
  | Empty_Error
  | Error_Item of Datamodel._Value

exception Xquery_error of xquery_error_msg

let raise_empty_error () =
  raise (Xquery_error Empty_Error)

let raise_item_error v =
  raise (Xquery_error (Error_Item v))

let downgrade_error msg =
  match msg with
  | Empty_Error ->
      raise (Query (Error ""))

  | Error_Item dm ->
      let errstr =
	try
	  if (sequenceIsEmpty dm) then 
	    ""
	  else
	    let i = (sequenceGetSingle dm) in
	    if (isNode i) then 
	      let n = getNode i in
	      nodeStringValue n
	    else 
	      Datatypes.erase_atomic_value (getAtomicValue i)
	with
	| _ ->
	    raise (Query (Internal_Error "xf:error function: argument not an item"))
      in
      raise (Query (Error errstr))

