(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                            XML Query Engine                         *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: overloaded.ml,v 1.1 2003/04/21 16:08:47 simeon Exp $ *)

open Namespace
open Xquery_ast

(* Mapping table for overloaded functions *)

let overloaded_functions = [
  ((op_uri,"numeric-add"), 2), [
    ((op_uri, "double-add"), 2); 
    ((op_uri, "float-add"), 2); 
    ((op_uri, "decimal-add"), 2); 
    ((op_uri, "integer-add"), 2);  
    ((op_uri, "int-add"), 2)]; 

  ((op_uri,"numeric-subtract"), 2), [
    ((op_uri, "double-subtract"), 2); 
    ((op_uri, "float-subtract"), 2); 
    ((op_uri, "decimal-subtract"), 2); 
    ((op_uri, "integer-subtract"), 2);  
    ((op_uri, "int-subtract"), 2)]; 

  ((op_uri,"numeric-multiply"), 2), [
    ((op_uri, "double-multiply"), 2); 
    ((op_uri, "float-multiply"), 2); 
    ((op_uri, "decimal-multiply"), 2); 
    ((op_uri, "integer-multiply"), 2); 
    ((op_uri, "int-multiply"), 2)]; 

  ((op_uri,"numeric-divide"), 2), [
    ((op_uri, "double-divide"), 2); 
    ((op_uri, "float-divide"), 2);  
    ((op_uri, "decimal-divide"), 2); 
    ((op_uri, "integer-divide"), 2);  
    ((op_uri, "int-divide"), 2)]; 

  ((op_uri,"numeric-mod"), 2), [ 
    ((op_uri, "double-mod"), 2); 
    ((op_uri, "float-mod"), 2); 
    ((op_uri, "decimal-mod"), 2); 
    ((op_uri, "integer-mod"), 2); 
    ((op_uri, "int-mod"), 2)]; 

  ((op_uri,"numeric-unary-plus"), 1), [ 
    ((op_uri,"double-unary-plus"), 1);
    ((op_uri,"float-unary-plus"), 1);
    ((op_uri,"decimal-unary-plus"), 1);
    ((op_uri,"integer-unary-plus"), 1); 
    ((op_uri,"int-unary-plus"), 1)];

  ((op_uri,"numeric-unary-minus"), 1), [
    ((op_uri,"double-unary-minus"), 1);
    ((op_uri,"float-unary-minus"), 1);
    ((op_uri,"decimal-unary-minus"), 1);
    ((op_uri,"integer-unary-minus"), 1); 
    ((op_uri,"int-unary-minus"), 1)];

  ((op_uri,"numeric-idivide"), 2), [
    ((op_uri, "integer-divide"), 2);  
    ((op_uri, "int-divide"), 2)]; 

  ((op_uri,"equal"), 2), [
    ((op_uri, "double-equal"), 2); 
    ((op_uri, "float-equal"), 2); 
    ((op_uri, "decimal-equal"), 2); 
    ((op_uri, "integer-equal"), 2); 
    ((op_uri, "int-equal"), 2); 
    ((op_uri, "string-equal"), 2); 
    ((op_uri, "QName-equal"), 2); 
    ((op_uri, "anyURI-equal"), 2); 
    ((op_uri, "boolean-equal"), 2)]; 

  ((op_uri,"nequal"), 2), [
    ((op_uri, "double-nequal"), 2); 
    ((op_uri, "float-nequal"), 2); 
    ((op_uri, "decimal-nequal"), 2); 
    ((op_uri, "integer-nequal"), 2); 
    ((op_uri, "int-nequal"), 2); 
    ((op_uri, "string-nequal"), 2); 
    ((op_uri, "QName-nequal"), 2); 
    ((op_uri, "anyURI-nequal"), 2); 
    ((op_uri, "boolean-nequal"), 2)]; 

  ((op_uri,"lt"), 2), [
    ((op_uri, "double-lt"), 2); 
    ((op_uri, "float-lt"), 2); 
    ((op_uri, "decimal-lt"), 2); 
    ((op_uri, "integer-lt"), 2); 
    ((op_uri, "int-lt"), 2); 
    ((op_uri, "string-lt"), 2); 
    ((op_uri, "boolean-lt"), 2)]; 

  ((op_uri,"gt"), 2), [
    ((op_uri, "double-gt"), 2); 
    ((op_uri, "float-gt"), 2); 
    ((op_uri, "decimal-gt"), 2); 
    ((op_uri, "integer-gt"), 2); 
    ((op_uri, "int-gt"), 2); 
    ((op_uri, "string-gt"), 2); 
    ((op_uri, "boolean-gt"), 2)]; 

  ((op_uri,"le"), 2), [
    ((op_uri, "double-le"), 2); 
    ((op_uri, "float-le"), 2); 
    ((op_uri, "decimal-le"), 2); 
    ((op_uri, "integer-le"), 2); 
    ((op_uri, "int-le"), 2); 
    ((op_uri, "string-le"), 2); 
    ((op_uri, "boolean-le"), 2)]; 

  ((op_uri,"ge"), 2), [
    ((op_uri, "double-ge"), 2); 
    ((op_uri, "float-ge"), 2); 
    ((op_uri, "decimal-ge"), 2); 
    ((op_uri, "integer-ge"), 2); 
    ((op_uri, "int-ge"), 2); 
    ((op_uri, "string-ge"), 2); 
    ((op_uri, "boolean-ge"), 2)]; 

  ((fn_uri,"avg"), 1), [
    ((fn_uri,"avg-double"), 1);
    ((fn_uri,"avg-float"), 1);
    ((fn_uri,"avg-decimal"), 1);
    ((fn_uri,"avg-integer"), 1);
    ((fn_uri,"avg-int"), 1)];

  ((fn_uri,"max"), 1), [
    ((fn_uri,"max-double"), 1);
    ((fn_uri,"max-float"), 1);
    ((fn_uri,"max-decimal"), 1);
    ((fn_uri,"max-integer"), 1);
    ((fn_uri,"max-int"), 1)];

  ((fn_uri, "min"), 1), [
    ((fn_uri,"min-double"), 1);
    ((fn_uri,"min-float"), 1);
    ((fn_uri,"min-decimal"), 1);
    ((fn_uri,"min-integer"), 1);
    ((fn_uri,"min-int"), 1)];

  ((fn_uri,"sum"), 1), [
    ((fn_uri,"sum-double"), 1);
    ((fn_uri,"sum-float"), 1);
    ((fn_uri,"sum-decimal"), 1);
    ((fn_uri,"sum-integer"), 1);
    ((fn_uri,"sum-int"), 1)];
] 

(* The internal hashtable for overloaded functions *)

(* Note:
     Those functions are built-in in the semantics of XQuery, so that
     is ok to leave them as a global variable in the system.
   - Jerome
   *)

type overloaded_functions_table_type =
  ((rfname * int), (rfname * int)) Hashtbl.t

let overloaded_functions_table =
  Hashtbl.create 167

let add_to_overloaded_functions_table fname1 fname2 =
  Hashtbl.add overloaded_functions_table fname1 fname2

let bulk_add_to_overloaded_functions_table (fname1,fnamelist2) =
  List.iter (add_to_overloaded_functions_table fname1) fnamelist2

let _ = 
  List.iter bulk_add_to_overloaded_functions_table overloaded_functions

(* Is a function overloaded ? *)

let is_overloaded fname =
  Hashtbl.mem overloaded_functions_table fname

(* What are all the functions a given overloaded function corresponds to? *)

let all_overloaded_bindings_for_function fname =
  Hashtbl.find_all overloaded_functions_table fname

let resolve_overloaded_function norm_context fname fi =
  let resolve_non_overloaded_name fname =
    (fst fname,Norm_context.one_sig_from_norm_context norm_context fname fi)
  in
  if is_overloaded fname
  then
    let all_bindings =
      all_overloaded_bindings_for_function fname
    in
    List.map resolve_non_overloaded_name all_bindings
  else
    [resolve_non_overloaded_name fname]

