(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                            XML Query Engine                         *)
(*                                                                     *)
(*  Copyright 2000, Lucent Technologies.                               *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: store.ml,v 1.5 2002/10/27 20:02:53 simeon Exp $ *)

open Error
open Namespace
open Datatypes
open Datamodel

(* Note:

     This module is used to 'store' document that have been opened by
     the query.

     For now, this is only keeping track of document access through
     one of the 'document()' functions, but this has the ambition to
     grow towards actual physical stores eventually.

   - Jerome
*) 

let store_in_file dname instance =
  let name =  dname ^ ".dm.marshal" in
  let oc = open_out_bin name in
  Marshal.to_channel oc instance [Marshal.Closures];
  close_out oc

let get_from_file dname =
  let name =  dname ^ ".dm.marshal" in
  let ic = open_in_bin name in
  let dm = (Marshal.from_channel ic : _Value) in
  begin
    close_in ic;
    dm
  end


(* Hashtable for opened documents *)

let alive_documents : (xs_string, _Value) Hashtbl.t = Hashtbl.create 137

let add_document dname instance =
(*  store_in_file dname instance; *)
  Hashtbl.add alive_documents dname instance

let lookup_document dname =
  try
    Some (Hashtbl.find alive_documents dname)
  with
  | _ ->
      None

(* Below is alpha code, used for projected documents.
   - Jerome and Amelie *)

(* There is just one global variable here for now - Jerome *)

let projections = ref []

let add_projections pr = projections := pr
let get_projections () = !projections

let alive_projected_documents : (xs_string, _Value) Hashtbl.t = Hashtbl.create 137

let add_projected_document dname instance =
(* store_in_file dname instance; *)
  Hashtbl.add alive_projected_documents dname instance

let lookup_projected_document dname =
  try
    Some (Hashtbl.find alive_projected_documents dname)
  with
  | _ ->
      None

