(* Module added by Trevor, for http get *)
(* Modified by Nicola Onose, May 2003 *)

module HTTP = struct
  exception Break
  let buflen = 8192
		 
  let gen_request h p s meth extra = (* host port resource *)
    let buf = String.create buflen in
    let longbuf = Buffer.create buflen in
    let targetInetAddr =
      try
        let targetHostEntry = Unix.gethostbyname h in
          targetHostEntry.Unix.h_addr_list.(0)
      with Not_found ->
        raise (Failure
                  (Printf.sprintf
                     "Can't find the IP address of the server (%s)" h))
     in

     let sock = Unix.socket Unix.PF_INET Unix.SOCK_STREAM 0 in
     Unix.connect sock (Unix.ADDR_INET(targetInetAddr,p));

     let request =
       match extra with 
	 | Some (headers, msg) ->
	     Printf.sprintf
	     "%s %s HTTP/1.1\r\nHost: %s:%d\r\nConnection: close\r\nContent-Length: %d\r\n%s\r\n%s\r\n\r\n"
             meth s h p (String.length msg) headers msg
	 | None ->
	     Printf.sprintf
	     "%s %s HTTP/1.1\r\nHost: %s:%d\r\nConnection: close\r\n\r\n"
             meth s h p
     in

     let w1 = Unix.write sock request 0 (String.length request) in
     (* Printf.printf "Receiving:\n"; *)
     (* let total_len = ref 0 in *)
     (try
       while true do
         (* Printf.printf "Receiving...\n"; flush(stdout); *)
         let len = Unix.read sock buf 0 buflen in
         if len <= 0 then raise Break;
         Buffer.add_substring longbuf buf 0 len;
         (*
            Printf.printf "%s" (String.escaped (String.sub buf 0 len));
            flush(stdout);
            total_len := len + !total_len *)
       done
     with Break -> ());
     (* Printf.printf "Total bytes received: %d\n" !total_len;
        flush(stdout); *)
     Unix.close sock;
     let result = Buffer.contents longbuf in
     let rec after_crlfcrlf start =
        let cr = String.index_from result start '\r' in
        if String.get result (cr+1) = '\n' &
           String.get result (cr+2) = '\r' &
           String.get result (cr+3) = '\n'
        then cr+4
        else after_crlfcrlf (cr+1) in
     try
        let x = after_crlfcrlf 0 in
        String.sub result x (String.length result - x)
     with _ -> "ERROR"

  let get h p s = gen_request h p s "GET" None

end

(* We would prefer to use the standard url parsing routines, but
    (1) base/uri.mli makes the type of _uri abstract so we can't
        use the functions in tools/netstring/neturl.ml
    (2) those functions don't seem to decode ldap urls properly,
        e.g., they complain on
     
ldap://ldap.itd.umich.edu:389/ 
o=University%20of%20Michigan,c=US??sub?sn=john
*)

let decode_url s =
   let len = String.length s in
   try
     let i = String.index s ':' in (* might raise Not_found *)
     let scheme = String.sub s 0 i in
     if i+3 >= len then raise Not_found;
     if String.get s (i+1) <> '/' then raise Not_found;
     if String.get s (i+2) <> '/' then raise Not_found;
     let colon_position =
       try Some(String.index_from s (i+3) ':')
       with Not_found -> None in
     let slash_position =
       try Some(String.index_from s (i+3) '/')
       with Not_found -> None in
     let host,port,after_port =
       match colon_position,slash_position with
         None,None ->
           String.sub s (i+3) (len-(i+3)),
           None,
           len
       | Some j,None ->
           String.sub s (i+3) (j-(i+3)),
           Some(String.sub s (j+1) (len-(j+1))),
           len
       | None,Some j ->
           String.sub s (i+3) (j-(i+3)),
           None,
           (j+1)
       | Some j,Some k ->
           String.sub s (i+3) (j-(i+3)),
           Some(String.sub s (j+1) (k-(j+1))),
           (k+1) in
     let rest =
       if after_port >= len then None else
       Some(String.sub s after_port (len-after_port)) in
     Some(scheme,host,port,rest)
   with Not_found -> None
          

