(***********************************************************************)
(*                                                                     *)
(*                               Galax                                 *)
(*                            XQuery Engine                            *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: mapschema.ml,v 1.10 2003/06/13 21:51:56 simeon Exp $ *)

open Format
open Print
open Error
open Xquery_ast

let prefix     = ref (Namespace.NSDefaultElementPrefix)
let typesyntax = ref false     (* True if using XQuery type syntax, false is using XML Schema *)
let output     = ref None      (* Set to a filename *)

let usage_msg = 
  sprintf "Usage: %s [-typesyntax] [-prefix prefix] [-output file] schemafile(s)" Sys.argv.(0)

let process_args () =
  let args = ref [] in
  Arg.parse
    [ "-prefix", Arg.String (fun f  -> prefix := (Namespace.NSPrefix f)), "Namespace prefix";
      "-output" , Arg.String (fun x -> output := Some x), "Writing mapped schema in file";
      "-typesyntax", Arg.Set typesyntax, "Uses XQuery type system syntax instead of XML Schema syntax" ]
    (fun arg -> args := arg :: !args) usage_msg;
  match !args with
    [] -> failwith "Input file not specified"
  | fnames ->
      fnames

let process_file schema_file =
  printf "---------------------------------------\n";
  printf "Loading XML Schema: %s\n" schema_file;
  printf "---------------------------------------\n";
  printf "%s" !Toputils.separator;
  flush stdout;
  begin
    try
      let (_, sdl) =
	if !typesyntax
	then
	  Toputils.convert_xquery_context schema_file
	else
	  Toputils.convert_schema schema_file !prefix
      in
      let oc = 
	match !output with
	| None -> stdout
	| Some f -> open_out f
      in
      let ff = Format.formatter_of_out_channel oc in
      begin
	fprintf ff "%s"
	  ("{!-- ---------------------------------------------------------------------------\n" ^
	   "\t\tGalax XML Schema Mapping Tool\n" ^
	   "\t\t\tXML Schema: " ^ schema_file ^ "\n" ^
	   "     --------------------------------------------------------------------------- --}\n\n");
	Toputils.print_schema_types ff sdl;
	fprintf ff "%s"
	  "\n\n{!-- For more information about Galax, go to http://db.bell-labs.com/galax/ --}\n";
	match !output with
	| None ->
	    ()
	| Some _ ->
	    close_out oc
      end
    with
    | e ->
	begin
	  eprintf_error "  " e;
	  fprintf err_formatter "@."
	end
  end;
  printf "%s" !Toputils.separator

let main () =
  let input_files = process_args() in
  List.iter process_file input_files

let exec func arg =
  try
    func arg
  with
  | e ->
      begin
	eprintf_error "  " e;
	fprintf err_formatter "@."
      end

let _ =
  exec main()
