(***********************************************************************)
(*                                                                     *)
(*                               Galax                                 *)
(*                            XQuery Engine                            *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: toputils.ml,v 1.55 2003/05/21 15:03:49 simeon Exp $ *)

(* Galax modules *)

open Conf
open Print
open Error
open Xquery_ast

(* Caml modules *)

open Format

(**********************)
(* Printing functions *)
(**********************)

(* Separator *)

let separator =
  ref "\n\n------------------------------------------------------------------------------\n\n"

(* Print the environment *)

let fprint_query_env ff q =
  fprintf ff "%s" !separator;
  fprintf ff  "Function definitions:\n";
  fprintf ff  "---------------------\n";
  List.iter (fprintf_cfunction_def ff "") q.pcmodule_functions;
  fprintf ff  "Global variables:\n";
  fprintf ff  "-----------------\n";
  List.iter (fprintf_cvar_decl ff "") q.pcmodule_vars;
  fprintf ff  "%s" !separator

(*********************)
(* Loading functions *)
(*********************)

(* Load stdlib *)

let load_pervasive () =
  if not(!stdlib)
  then
    Xquery_astutil.empty_query_module
  else
    begin
      if !debugging then
	begin
	  printf "Parsing pervasives\n";
	  flush stdout
	end;
      let query =
	try
	  Parseutil.parseStringFXQuery !pervasive_content
	with
	| (Sys_error _) ->
	    (try
	      Parseutil.parseFXQuery "pervasive.xq" 
	    with
	    | (Sys_error _) ->
		raise (Query (Parsing (Finfo.bogus, "Cannot load pervasive.xq"))))
      in 
      query
    end

(*********)
(* Setup *)
(*********)

let setup margin indent =
  set_max_boxes 1000;
  set_margin margin;
  set_max_indent indent

(*********************)
(* Schema operations *)
(*********************)

let convert_schema f prefix =
  let nse = Namespace.empty_nsenv () in
  let xml_schema = Parseutil.parseXMLSchema f in
  let (ncn, uri, types_in_schema) = Schemamapping.map_xml_schema prefix xml_schema in
  let nse' = Namespace.add_schema_ns (Namespace.add_ns nse ncn uri) uri in
  (nse', types_in_schema)

let convert_xquery_context context_file =
  let qml = Parseutil.parseFXQuery context_file in
  let sdl = qml.pmodule_sorts
  and cdl = qml.pmodule_contexts in
  let (nse, impt_types) = Mapping.map_context cdl in
  let all_sort_decl = impt_types @ sdl in
  (nse, all_sort_decl)

let load_schema (nse, sdl) =
  Mapping.map_sort nse sdl

let print_schema_types ff sdl =
  let printed_form = Print.bprintf_schema "" sdl in
  fprintf ff "%s" printed_form

